@@ -2,7 +2,6 @@
 #
 # SPDX-License-Identifier: GPL-3.0-only
 
-from typing import Any
 from typing import TextIO
 
 import cProfile
@@ -40,7 +39,6 @@ from gajim.common.events import AllowGajimUpdateCheck
 from gajim.common.events import DBMigrationError
 from gajim.common.events import GajimUpdateAvailable
 from gajim.common.events import SignedIn
-from gajim.common.settings import LegacyConfig
 from gajim.common.settings import Settings
 from gajim.common.storage.archive.storage import MessageArchiveStorage
 from gajim.common.storage.cache import CacheStorage
@@ -64,6 +62,7 @@ class CoreApplication(ged.EventHelper):
         app.app = self
         app.print_version()
         app.detect_dependencies()
+        app.init_process_pool()
         configpaths.create_paths()
 
         app.settings = Settings()
@@ -71,7 +70,6 @@ class CoreApplication(ged.EventHelper):
 
         passwords.init()
 
-        app.config = LegacyConfig()
         app.commands = ChatCommands()
 
         try:
@@ -93,8 +91,8 @@ class CoreApplication(ged.EventHelper):
         app.cert_store = CertificateStore()
         app.task_manager = TaskManager()
 
-        from gajim.common.call_manager import CallManager
-        app.call_manager = CallManager()
+        # from gajim.common.call_manager import CallManager
+        # app.call_manager = CallManager()
 
         from gajim.common.preview import PreviewManager
         app.preview_manager = PreviewManager()
@@ -216,9 +214,7 @@ class CoreApplication(ged.EventHelper):
         ps = ps.sort_stats(SortKey.TIME)
         ps.print_stats()
 
-    def start_shutdown(self, *args: Any, **kwargs: Any) -> None:
-        app.app.systray.shutdown()
-
+    def start_shutdown(self) -> None:
         accounts_to_disconnect: dict[str, Client] = {}
 
         for client in app.get_clients():
@@ -240,15 +236,18 @@ class CoreApplication(ged.EventHelper):
                                        _on_disconnect)
 
         for client in accounts_to_disconnect.values():
-            client.change_status('offline', kwargs.get('message', ''))
+            client.change_status('offline', '')
 
     def _shutdown_core(self) -> None:
         # Commit any outstanding SQL transactions
+        app.process_pool.shutdown(cancel_futures=True)
         app.storage.archive.cleanup_chat_history()
         app.storage.cache.shutdown()
         app.storage.archive.shutdown()
         app.settings.shutdown()
+
         self.end_profiling()
+        app.app.systray.shutdown()
         configpaths.cleanup_temp()
         logind.shutdown()
 
@@ -259,7 +258,6 @@ class CoreApplication(ged.EventHelper):
     def _show_warnings() -> None:
         import traceback
         import warnings
-        os.environ['GAJIM_LEAK'] = 'true'
 
         def warn_with_traceback(message: Warning | str,
                                 category: type[Warning],
@@ -315,6 +313,7 @@ class CoreApplication(ged.EventHelper):
 
     def check_for_gajim_updates(self) -> None:
         self._log.info('Checking for Gajim updates')
+
         request = create_http_request()
         request.send('GET', 'https://gajim.org/current-version.json',
                      callback=self._on_update_response)
@@ -411,8 +410,6 @@ class CoreApplication(ged.EventHelper):
 
         app.get_client(account).cleanup()
         del app.connections[account]
-        if account in app.interface.instances:
-            del app.interface.instances[account]
         del app.nicks[account]
         del app.to_be_removed[account]
 
