@@ -23,6 +23,7 @@ from nbxmpp.protocol import JID
 from gajim.common import app
 from gajim.common import modules
 from gajim.common import passwords
+from gajim.common.client_modules import ClientModules
 from gajim.common.const import ClientState
 from gajim.common.const import SimpleClientState
 from gajim.common.events import AccountConnected
@@ -40,8 +41,11 @@ from gajim.common.helpers import get_resource
 from gajim.common.helpers import Observable
 from gajim.common.helpers import warn_about_plain_connection
 from gajim.common.i18n import _
+from gajim.common.i18n import get_rfc5646_lang
 from gajim.common.idle import IdleMonitorManager
 from gajim.common.idle import Monitor
+from gajim.common.modules.contacts import BareContact
+from gajim.common.modules.message import build_message_stanza
 from gajim.common.structs import OutgoingMessage
 from gajim.common.util.http import create_http_session
 from gajim.common.util.status import get_idle_status_message
@@ -64,9 +68,10 @@ def call_counter(func: Any):
     return helper
 
 
-class Client(Observable):
+class Client(Observable, ClientModules):
     def __init__(self, account: str) -> None:
         Observable.__init__(self, log)
+        ClientModules.__init__(self, account)
         self._client = None
         self._account = account
         self.name = account
@@ -107,11 +112,8 @@ class Client(Observable):
             self._screensaver_handler_id = app.app.connect(
                 'notify::screensaver-active', self._screensaver_state_changed)
 
-    def get_module(self, name: modules.ModulesLiteralT) -> modules.ModulesT:
-        return modules.get_module(self._account, name)
-
     def _set_state(self, state: ClientState) -> None:
-        log.info('State: %s', state)
+        log.info('State: %s', repr(state))
         self._state = state
 
     @property
@@ -169,6 +171,7 @@ class Client(Observable):
 
         self._client = NBXMPPClient(log_context=self._account)
         self.connection = self._client
+        self._client.set_lang(get_rfc5646_lang())
         self._client.set_domain(self._address.domain)
         self._client.set_username(self._address.localpart)
         self._client.set_resource(get_resource(self._account))
@@ -278,12 +281,21 @@ class Client(Observable):
                                                  'resource',
                                                  'gajim.$rand')
             if error == 'system-shutdown':
+                account_label = app.settings.get_account_setting(
+                    self._account, "account_label"
+                )
+                account_text = _("Account: %s") % account_label
+                shutdown_text = text or _("The server was shut down.")
+
                 app.ged.raise_event(
-                    Notification(context_id="",
-                                 account=self._account,
-                                 type='server-shutdown',
-                                 title=_('Server Shutdown'),
-                                 text=text or _('The server was shut down.')))
+                    Notification(
+                        context_id="",
+                        account=self._account,
+                        type="server-shutdown",
+                        title=_("Server Shutdown"),
+                        text=f"{shutdown_text}\n{account_text}",
+                    )
+                )
 
         elif domain == StreamError.SASL:
             self._reconnect = False
@@ -358,6 +370,12 @@ class Client(Observable):
         own_jid = self.get_own_jid()
         return own_jid.bare_match(jid)
 
+    def get_own_contact(self) -> BareContact:
+        jid = self.get_own_jid()
+        contact = self.get_module("Contacts").get_contact(jid)
+        assert isinstance(contact, BareContact)
+        return contact
+
     def get_own_jid(self) -> JID:
         '''
         Return the last full JID we received on a bind event.
@@ -492,7 +510,7 @@ class Client(Observable):
             log.warning('Trying to send message while offline')
             return
 
-        stanza = self.get_module('Message').build_message_stanza(message)
+        stanza = build_message_stanza(message, self.get_own_jid())
         message.set_stanza(stanza)
 
         method = message.contact.settings.get('encryption')
