@@ -1,38 +1,26 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
-from typing import Any
-from typing import Optional
-from typing import Union
 from typing import NamedTuple
 
-from enum import IntEnum
+from dataclasses import dataclass
+from datetime import datetime
+from datetime import UTC
 from enum import Enum
+from enum import IntEnum
 from enum import unique
 from functools import total_ordering
 
 from gi.repository import Gio
-
-from nbxmpp.namespaces import Namespace
 from nbxmpp.const import PresenceShow
+from nbxmpp.namespaces import Namespace
 from nbxmpp.protocol import JID
 
 from gajim.common.i18n import _
-from gajim.common.i18n import Q_
-
+from gajim.common.i18n import p_
 
 STOP_EVENT = True
 PROPAGATE_EVENT = False
@@ -40,8 +28,16 @@ PROPAGATE_EVENT = False
 MAX_MESSAGE_CORRECTION_DELAY = 300
 
 
-class EncryptionData(NamedTuple):
-    additional_data: Any = None
+class EncryptionInfoMsg(Enum):
+    BAD_OMEMO_CONFIG = _('This chat’s configuration is unsuitable for '
+                         'encryption with OMEMO. To use OMEMO in this chat, '
+                         'it should be non-anonymous and members-only.')
+    NO_FINGERPRINTS = _('To send an encrypted message, you have to decide '
+                        'whether to trust the device of your contact.')
+    QUERY_DEVICES = _('No devices found to encrypt this message to. '
+                      'Querying for devices now…')
+    UNDECIDED_FINGERPRINTS = _('There are devices for which you have not made '
+                               'a trust decision yet.')
 
 
 class Entity(NamedTuple):
@@ -59,25 +55,30 @@ class RowHeaderType(IntEnum):
 class AvatarSize(IntEnum):
     TAB = 16
     SHOW_CIRCLE = 24
+    SMALL = 24
     ROSTER = 32
+    START_CHAT = 32
     VCARD_HEADER = 40
-    ACCOUNT_SIDE_BAR = 40
-    WORKSPACE = 40
+    ACCOUNT_SIDE_BAR = 24
+    ACCOUNT_SIDE_BAR_WARNING = 20
+    WORKSPACE = 36
     WORKSPACE_EDIT = 100
     CHAT = 48
-    NOTIFICATION = 48
+    MESSAGE_REPLY = 24
+    NOTIFICATION = 96
     CALL = 100
     CALL_BIG = 200
     GROUP_INFO = 100
     TOOLTIP = 100
     ACCOUNT_PAGE = 150
+    ACTIVITY_PAGE = 100
     VCARD = 200
     PUBLISH = 200
 
 
-class ArchiveState(IntEnum):
-    NEVER = 0
-    ALL = 1
+class ArchiveState:
+    NEVER = None
+    ALL = datetime(1970, 1, 1, tzinfo=UTC)
 
 
 @unique
@@ -85,6 +86,7 @@ class PathLocation(IntEnum):
     CONFIG = 0
     CACHE = 1
     DATA = 2
+    NONE = 3
 
 
 @unique
@@ -94,58 +96,6 @@ class PathType(IntEnum):
     FOLDER_OPTIONAL = 2
 
 
-@unique
-class KindConstant(IntEnum):
-    STATUS = 0
-    GCSTATUS = 1
-    GC_MSG = 2
-    SINGLE_MSG_RECV = 3
-    CHAT_MSG_RECV = 4
-    SINGLE_MSG_SENT = 5
-    CHAT_MSG_SENT = 6
-    ERROR = 7
-    FILE_TRANSFER_INCOMING = 8
-    FILE_TRANSFER_OUTGOING = 9
-    CALL_INCOMING = 10
-    CALL_OUTGOING = 11
-
-    def __str__(self):
-        return str(self.value)
-
-
-@unique
-class TypeConstant(IntEnum):
-    AIM = 0
-    GG = 1
-    HTTP_WS = 2
-    ICQ = 3
-    MSN = 4
-    QQ = 5
-    SMS = 6
-    SMTP = 7
-    TLEN = 8
-    YAHOO = 9
-    NEWMAIL = 10
-    RSS = 11
-    WEATHER = 12
-    MRIM = 13
-    NO_TRANSPORT = 14
-
-
-@unique
-class SubscriptionConstant(IntEnum):
-    NONE = 0
-    TO = 1
-    FROM = 2
-    BOTH = 3
-
-
-@unique
-class JIDConstant(IntEnum):
-    NORMAL_TYPE = 0
-    ROOM_TYPE = 1
-
-
 @unique
 class StyleAttr(Enum):
     COLOR = 'color'
@@ -155,7 +105,7 @@ class StyleAttr(Enum):
 
 @unique
 class CSSPriority(IntEnum):
-    PRE_APPLICATION = 599
+    PRE_THEME = 199
     APPLICATION = 600
     APPLICATION_DARK = 601
     DEFAULT_THEME = 610
@@ -215,17 +165,6 @@ class Display(Enum):
     QUARTZ = 'GdkQuartzDisplay'
 
 
-class URIType(Enum):
-    INVALID = 'invalid'
-    XMPP = 'xmpp'
-    MAIL = 'mail'
-    GEO = 'geo'
-    WEB = 'web'
-    FILE = 'file'
-    AT = 'at'
-    TEL = 'tel'
-
-
 # https://xmpp.org/registrar/querytypes.html
 class XmppUriQuery(Enum):
     NONE = ''
@@ -233,21 +172,29 @@ class XmppUriQuery(Enum):
     JOIN = 'join'
 
     @staticmethod
-    def from_str(s: str) -> Optional[XmppUriQuery]:
+    def from_str(s: str) -> XmppUriQuery | None:
         try:
             return XmppUriQuery(s)
         except ValueError:
             return None
 
+    @staticmethod
+    def from_str_or_none(s: str) -> XmppUriQuery:
+        qtype = XmppUriQuery.from_str(s)
+        if qtype:
+            return qtype
+        return XmppUriQuery.NONE
+
 
 class MUCJoinedState(Enum):
     JOINED = 'joined'
     NOT_JOINED = 'not joined'
+    BEFORE_JOINING = 'before joining'
     JOINING = 'joining'
     CREATING = 'creating'
     CAPTCHA_REQUEST = 'captcha in progress'
     CAPTCHA_FAILED = 'captcha failed'
-    PASSWORD_REQUEST = 'password request'
+    PASSWORD_REQUEST = 'password request'  # noqa: S105
 
     def __str__(self):
         return self.name
@@ -264,6 +211,10 @@ class MUCJoinedState(Enum):
     def is_joining(self):
         return self == MUCJoinedState.JOINING
 
+    @property
+    def is_before_joining(self):
+        return self == MUCJoinedState.BEFORE_JOINING
+
     @property
     def is_creating(self):
         return self == MUCJoinedState.CREATING
@@ -353,22 +304,28 @@ class CallType(Enum):
     VIDEO = 'video'
 
 
+@dataclass
+class Draft:
+    text: str
+    reply_pk: int | None
+
+
 MUC_CREATION_EXAMPLES = [
-    (Q_('?Group chat name:Team'),
-     Q_('?Group chat description:Project discussion'),
-     Q_('?Group chat address:team')),
-    (Q_('?Group chat name:Family'),
-     Q_('?Group chat description:Spring gathering'),
-     Q_('?Group chat address:family')),
-    (Q_('?Group chat name:Vacation'),
-     Q_('?Group chat description:Trip planning'),
-     Q_('?Group chat address:vacation')),
-    (Q_('?Group chat name:Repairs'),
-     Q_('?Group chat description:Local help group'),
-     Q_('?Group chat address:repairs')),
-    (Q_('?Group chat name:News'),
-     Q_('?Group chat description:Local news and reports'),
-     Q_('?Group chat address:news')),
+    (p_('Group chat name', 'Team'),
+     p_('Group chat description', 'Project discussion'),
+     p_('Group chat address', 'team')),
+    (p_('Group chat name', 'Family'),
+     p_('Group chat description', 'Spring gathering'),
+     p_('Group chat address', 'family')),
+    (p_('Group chat name', 'Vacation'),
+     p_('Group chat description', 'Trip planning'),
+     p_('Group chat address', 'vacation')),
+    (p_('Group chat name', 'Repairs'),
+     p_('Group chat description', 'Local help group'),
+     p_('Group chat address', 'repairs')),
+    (p_('Group chat name', 'News'),
+     p_('Group chat description', 'Local news and reports'),
+     p_('Group chat address', 'news')),
 ]
 
 
@@ -532,15 +489,16 @@ ARTISTS = [
 ]
 
 
-DEVS_CURRENT = [
-    'Yann Leboulanger (asterix AT lagaule.org)',
+MAINTAINERS = [
     'Philipp Hörist (philipp AT hoerist.com)',
     'Daniel Brötzmann (wurstsalat AT posteo.de)',
-    'André Apitzsch',
 ]
 
 
-DEVS_PAST = [
+DEVELOPERS = [
+    'Marc Henning',
+    'André Apitzsch',
+    'Yann Leboulanger (asterix AT lagaule.org)',
     'Stefan Bethge (stefan AT lanpartei.de)',
     'Alexander Cherniuk (ts33kr AT gmail.com)',
     'Stephan Erb (steve-e AT h3c.de)',
@@ -902,11 +860,17 @@ COMMON_FEATURES = [
     Namespace.HASHES_BLAKE2B_256,
     Namespace.HASHES_BLAKE2B_512,
     Namespace.JINGLE,
-    Namespace.JINGLE_FILE_TRANSFER_5,
-    Namespace.JINGLE_BYTESTREAM,
-    Namespace.JINGLE_IBB,
+    # TODO Jingle FT
+    # Namespace.JINGLE_FILE_TRANSFER_5,
+    # Namespace.JINGLE_BYTESTREAM,
+    # Namespace.JINGLE_IBB,
     Namespace.AVATAR_METADATA + '+notify',
-    Namespace.MESSAGE_MODERATE
+    Namespace.MESSAGE_MODERATE,
+    Namespace.REPLY,
+    Namespace.OMEMO_TEMP_DL + '+notify',
+    Namespace.STYLING,
+    Namespace.REACTIONS,
+    Namespace.MDS + '+notify',
 ]
 
 
@@ -921,30 +885,36 @@ SHOW_LIST = [
 ]
 
 SHOW_STRING = {
-    'dnd': _('Busy'),
-    'xa': _('Not Available'),
-    'chat': _('Free for Chat'),
-    'online': Q_('?user status:Available'),
+    'dnd': p_('User status', 'Busy'),
+    'xa': p_('User status', 'Not Available'),
+    'chat': p_('User status', 'Free for Chat'),
+    'online': p_('User status', 'Available'),
     'connecting': _('Connecting'),
-    'away': _('Away'),
+    'away': p_('User status', 'Away'),
     'offline': _('Offline'),
 }
 
 SHOW_STRING_MNEMONIC = {
-    'dnd': _('_Busy'),
-    'xa': _('_Not Available'),
-    'chat': _('_Free for Chat'),
-    'online': Q_('?user status:_Available'),
+    'dnd': p_('User status', '_Busy'),
+    'xa': p_('User status', '_Not Available'),
+    'chat': p_('User status', '_Free for Chat'),
+    'online': p_('User status', '_Available'),
     'connecting': _('Connecting'),
-    'away': _('A_way'),
+    'away': p_('User status', 'A_way'),
     'offline': _('_Offline'),
 }
 
 GAJIM_FAQ_URI = 'https://dev.gajim.org/gajim/gajim/wikis/help/gajimfaq'
+GAJIM_PRIVACY_POLICY_URI = 'https://gajim.org/privacy/'
 GAJIM_WIKI_URI = 'https://dev.gajim.org/gajim/gajim/wikis'
 GAJIM_SUPPORT_JID = 'gajim@conference.gajim.org'
 
 
+# Clickable URI schemes not (yet) registered with IANA (see ./iana.py)
+NONREGISTERED_URI_SCHEMES = {
+    'gemini',
+}
+
 # This is an excerpt of Media Types from
 # https://www.iana.org/assignments/media-types/media-types.xhtml
 # plus some additions
@@ -997,14 +967,22 @@ MIME_TYPES = (
     'audio/aac',
     'audio/ac3',
     'audio/flac',
+    'audio/mp3',
     'audio/mp4',
     'audio/mpeg',
     'audio/ogg',
     'audio/opus',
+    'audio/vnd.wave',
     'audio/wav',
+    'audio/wave',
+    'audio/x-aac',
     'audio/x-flac',
     'audio/x-m4a',
     'audio/x-matroska',
+    'audio/x-mp3',
+    'audio/x-mpeg',
+    'audio/x-ogg',
+    'audio/x-wav',
     # font/*
     'font/ttf',
     'font/woff',
@@ -1062,7 +1040,7 @@ TRUST_SYMBOL_DATA = {
                       _('Trust Not Decided'),
                       'warning-color'),
     Trust.BLIND: ('security-medium-symbolic',
-                  _('Unverified'),
+                  _('Blind Trust'),
                   'encrypted-color'),
     Trust.VERIFIED: ('security-high-symbolic',
                      _('Verified'),
@@ -1094,7 +1072,7 @@ class PresenceShowExt(Enum):
     def is_offline(self):
         return self == PresenceShowExt.OFFLINE
 
-    def __lt__(self, other: Union[PresenceShowExt, PresenceShow]) -> bool:
+    def __lt__(self, other: PresenceShowExt | PresenceShow) -> bool:
         if isinstance(other, PresenceShowExt):
             return False
         if not isinstance(other, PresenceShow):  # pyright: ignore
@@ -1109,3 +1087,7 @@ TLS_VERSION_STRINGS = {
     4: '1.2',
     5: '1.3',
 }
+
+
+RETRACTION_FALLBACK = (
+    "/me retracted a previous message, but it's unsupported by your client.")
