@@ -39,6 +39,7 @@ import precis_i18n.codec  # noqa: F401
 import qrcode
 from cryptography import x509
 from cryptography.hazmat.backends import default_backend
+from gi.repository import Gdk
 from gi.repository import GdkPixbuf
 from gi.repository import Gio
 from gi.repository import GLib
@@ -122,7 +123,7 @@ def get_contact_dict_for_account(account: str) -> dict[str, types.BareContact]:
     return contacts_dict
 
 
-def generate_qr_code(content: str) -> GdkPixbuf.Pixbuf | None:
+def generate_qr_code(content: str) -> Gdk.Texture:
     qr = qrcode.QRCode(version=None,
                        error_correction=qrcode.constants.ERROR_CORRECT_L,
                        box_size=6,
@@ -131,10 +132,11 @@ def generate_qr_code(content: str) -> GdkPixbuf.Pixbuf | None:
     qr.make(fit=True)
 
     img = qr.make_image(image_factory=QrcPilImage).convert('RGB')
-    return GdkPixbuf.Pixbuf.new_from_bytes(
+    pixbuf = GdkPixbuf.Pixbuf.new_from_bytes(
         GLib.Bytes.new(img.tobytes()),
         GdkPixbuf.Colorspace.RGB, False, 8,
         img.width, img.height, img.width*3)
+    return Gdk.Texture.new_for_pixbuf(pixbuf)
 
 
 def play_sound(sound_event: str,
@@ -177,39 +179,6 @@ def check_soundfile_path(file_: str,
     return None
 
 
-def strip_soundfile_path(file_: Path | str,
-                         dirs: list[Path] | None = None,
-                         abs_: bool = True):
-    '''
-    Remove knowns paths from a sound file
-
-    Filechooser returns an absolute path.
-    If path is a known fallback path, we remove it.
-    So config has no hardcoded path to DATA_DIR and text in textfield is
-    shorther.
-    param: file_: the filename to strip
-    param: dirs: list of knowns paths from which the filename should be stripped
-    param: abs_: force absolute path on dirs
-    '''
-
-    if not file_:
-        return None
-
-    if dirs is None:
-        dirs = [configpaths.get('MY_DATA'),
-                configpaths.get('DATA')]
-
-    file_ = Path(file_)
-    name = file_.name
-    for dir_ in dirs:
-        dir_ = dir_ / 'sounds' / name
-        if abs_:
-            dir_ = dir_.absolute()
-        if file_ == dir_:
-            return name
-    return file_
-
-
 def play_sound_file(str_path_to_soundfile: str, loop: bool = False) -> None:
     path_to_soundfile = check_soundfile_path(str_path_to_soundfile)
     if path_to_soundfile is None:
@@ -285,7 +254,7 @@ def get_global_proxy() -> ProxyData | None:
     return get_proxy(proxy_name)
 
 
-def get_account_proxy(account: str, fallback=True) -> ProxyData | None:
+def get_account_proxy(account: str, fallback: bool = True) -> ProxyData | None:
     proxy_name = app.settings.get_account_setting(account, 'proxy')
     if proxy_name:
         return get_proxy(proxy_name)
@@ -533,11 +502,10 @@ def write_file_async(
                       None,
                       _on_file_created)
 
-
 def load_file_async(path: Path,
                     callback: Callable[[bytes | None,
                                         GLib.Error | None,
-                                        Any | None], Any],
+                                        Any], Any],
                     user_data: Any | None = None) -> None:
 
     def _on_load_finished(file: Gio.File,
@@ -548,7 +516,8 @@ def load_file_async(path: Path,
         except GLib.Error as error:
             callback(None, error, user_data)
         else:
-            callback(contents, None, user_data)
+            # "contents" may be an empty bytes object
+            callback(contents or None, None, user_data)
 
     file = Gio.File.new_for_path(str(path))
     file.load_contents_async(None, _on_load_finished)
