@@ -58,25 +58,8 @@ def is_rtl_text(text: str) -> bool:
     return False
 
 
-def ngettext(s_sing: str,
-             s_plural: str,
-             n: int,
-             replace_sing: str | None = None,
-             replace_plural: str | None = None) -> str:
-    '''
-    Use as:
-        i18n.ngettext(
-            'leave room %s', 'leave rooms %s', len(rooms), 'a', 'a, b, c')
-
-    In other words this is a hack to ngettext() to support %s %d etc..
-    '''
-    text = _trans.translation.ngettext(s_sing, s_plural, n)
-    if n == 1 and replace_sing is not None:
-        return text % replace_sing
-
-    if n > 1 and replace_plural is not None:
-        return text % replace_plural
-    return text
+def ngettext(s_sing: str, s_plural: str, n: int) -> str:
+    return _trans.translation.ngettext(s_sing, s_plural, n)
 
 
 class Translation:
@@ -111,7 +94,7 @@ class Translation:
         if sys.platform == 'darwin':
             return self._get_darwin_default_lang()
 
-        return locale.getdefaultlocale()[0] or 'en'
+        return locale.getlocale()[0] or 'en'
 
     def init(self) -> None:
         try:
@@ -134,6 +117,7 @@ class Translation:
             self.translation = gettext.translation(DOMAIN, locale_dir)
             if hasattr(locale, 'bindtextdomain'):
                 locale.bindtextdomain(DOMAIN, locale_dir)
+                locale.textdomain(DOMAIN)
         except OSError:
             pass
 
