@@ -3,35 +3,38 @@
 #
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 # XEP-0115: Entity Capabilities
 
+from __future__ import annotations
+
+from typing import Any
+from typing import cast
+
 import weakref
 from collections import defaultdict
+from collections.abc import Callable
 
-from nbxmpp.namespaces import Namespace
-from nbxmpp.structs import StanzaHandler
-from nbxmpp.structs import DiscoIdentity
-from nbxmpp.util import compute_caps_hash
 from nbxmpp.errors import StanzaError
+from nbxmpp.namespaces import Namespace
+from nbxmpp.protocol import JID
+from nbxmpp.protocol import Presence
+from nbxmpp.structs import DiscoIdentity
+from nbxmpp.structs import DiscoInfo
+from nbxmpp.structs import PresenceProperties
+from nbxmpp.structs import StanzaHandler
+from nbxmpp.task import Task as nbxmpp_Task
+from nbxmpp.util import compute_caps_hash
 
 from gajim.common import app
+from gajim.common import types
 from gajim.common.const import COMMON_FEATURES
 from gajim.common.const import Entity
 from gajim.common.helpers import get_optional_features
-from gajim.common.task_manager import Task
 from gajim.common.modules.base import BaseModule
+from gajim.common.modules.contacts import GroupchatContact
+from gajim.common.task_manager import Task
 
 
 class Caps(BaseModule):
@@ -42,12 +45,13 @@ class Caps(BaseModule):
         'set_caps'
     ]
 
-    def __init__(self, con):
+    def __init__(self, con: types.Client) -> None:
         BaseModule.__init__(self, con)
 
         self.handlers = [
             StanzaHandler(name='presence',
                           callback=self._entity_caps,
+                          typ='available',
                           ns=Namespace.CAPS,
                           priority=51),
         ]
@@ -56,10 +60,11 @@ class Caps(BaseModule):
             DiscoIdentity(category='client', type='pc', name='Gajim')
         ]
 
-        self._queued_tasks_by_hash = defaultdict(set)
-        self._queued_tasks_by_jid = {}
+        self._queued_tasks_by_hash: defaultdict[
+            str, set[EntityCapsTask]] = defaultdict(set)
+        self._queued_tasks_by_jid: dict[JID, EntityCapsTask] = {}
 
-    def _queue_task(self, task):
+    def _queue_task(self, task: EntityCapsTask) -> None:
         old_task = self._get_task(task.entity.jid)
         if old_task is not None:
             self._remove_task(old_task)
@@ -69,26 +74,28 @@ class Caps(BaseModule):
         self._queued_tasks_by_jid[task.entity.jid] = task
         app.task_manager.add_task(task)
 
-    def _get_task(self, jid):
+    def _get_task(self, jid: JID) -> EntityCapsTask | None:
         return self._queued_tasks_by_jid.get(jid)
 
-    def _get_similar_tasks(self, task):
+    def _get_similar_tasks(self, task: EntityCapsTask) -> set[EntityCapsTask]:
         return self._queued_tasks_by_hash.pop(task.entity.hash)
 
-    def _remove_task(self, task):
+    def _remove_task(self, task: EntityCapsTask) -> None:
         task.set_obsolete()
         del self._queued_tasks_by_jid[task.entity.jid]
         self._queued_tasks_by_hash[task.entity.hash].discard(task)
 
-    def _remove_all_tasks(self):
+    def _remove_all_tasks(self) -> None:
         for task in self._queued_tasks_by_jid.values():
             task.set_obsolete()
         self._queued_tasks_by_jid.clear()
         self._queued_tasks_by_hash.clear()
 
-    def _entity_caps(self, _con, _stanza, properties):
-        if properties.type.is_error or properties.type.is_unavailable:
-            return
+    def _entity_caps(self,
+                     _con: types.NBXMPPClient,
+                     _stanza: Presence,
+                     properties: PresenceProperties
+                     ) -> None:
 
         if properties.is_self_presence:
             return
@@ -96,6 +103,10 @@ class Caps(BaseModule):
         if properties.entity_caps is None:
             return
 
+        if properties.muc_user is not None:
+            # Don’t query MUC participants
+            return
+
         task = EntityCapsTask(self._account, properties, self._execute_task)
 
         self._log.info('Received %s', task.entity)
@@ -106,6 +117,7 @@ class Caps(BaseModule):
             self._queue_task(task)
             return
 
+        assert properties.jid is not None
         app.storage.cache.set_last_disco_info(properties.jid,
                                               disco_info,
                                               cache_only=True)
@@ -113,7 +125,7 @@ class Caps(BaseModule):
         contact = self._con.get_module('Contacts').get_contact(properties.jid)
         contact.notify('caps-update')
 
-    def _execute_task(self, task):
+    def _execute_task(self, task: EntityCapsTask) -> None:
         self._log.info('Request %s from %s', task.entity.hash, task.entity.jid)
         self._con.get_module('Discovery').disco_info(
             task.entity.jid,
@@ -121,7 +133,7 @@ class Caps(BaseModule):
             callback=self._on_disco_info,
             user_data=task.entity.jid)
 
-    def _on_disco_info(self, nbxmpp_task):
+    def _on_disco_info(self, nbxmpp_task: nbxmpp_Task) -> None:
         jid = nbxmpp_task.get_user_data()
         task = self._get_task(jid)
         if task is None:
@@ -131,7 +143,7 @@ class Caps(BaseModule):
         self._remove_task(task)
 
         try:
-            disco_info = nbxmpp_task.finish()
+            disco_info = cast(DiscoInfo, nbxmpp_task.finish())
         except StanzaError as error:
             self._log.warning(error)
             return
@@ -145,10 +157,13 @@ class Caps(BaseModule):
                               disco_info.jid, error)
             return
 
+        assert disco_info.jid is not None
+        caps_hash = disco_info.get_caps_hash()
+        assert caps_hash is not None
         app.storage.cache.add_caps_entry(
             disco_info.jid,
             task.entity.method,
-            disco_info.get_caps_hash(),
+            caps_hash,
             disco_info)
 
         self._log.info('Finished query for %s', task.entity.hash)
@@ -160,9 +175,12 @@ class Caps(BaseModule):
             self._log.info('Update %s', task.entity.jid)
             contact = self._con.get_module('Contacts').get_contact(
                 task.entity.jid)
+            app.storage.cache.set_last_disco_info(task.entity.jid,
+                                                  disco_info,
+                                                  cache_only=True)
             contact.notify('caps-update')
 
-    def update_caps(self):
+    def update_caps(self) -> None:
         if not app.account_is_connected(self._account):
             return
 
@@ -174,21 +192,27 @@ class Caps(BaseModule):
         if not app.account_is_available(self._account):
             return
 
-        app.connections[self._account].change_status(
-            app.connections[self._account].status,
-            app.connections[self._account].status_message)
+        client = app.get_client(self._account)
+        client.change_status(client.status,
+                             client.status_message)
 
-    def cleanup(self):
-        self._remove_all_tasks()
+    def cleanup(self) -> None:
         BaseModule.cleanup(self)
+        self._remove_all_tasks()
 
 
 class EntityCapsTask(Task):
-    def __init__(self, account, properties, callback):
+    def __init__(self,
+                 account: str,
+                 properties: PresenceProperties,
+                 callback: Callable[..., Any]
+                 ) -> None:
         Task.__init__(self)
         self._account = account
         self._callback = weakref.WeakMethod(callback)
 
+        assert properties.jid is not None
+        assert properties.entity_caps is not None
         self.entity = Entity(jid=properties.jid,
                              node=properties.entity_caps.node,
                              hash=properties.entity_caps.ver,
@@ -196,12 +220,12 @@ class EntityCapsTask(Task):
 
         self._from_muc = properties.from_muc
 
-    def execute(self):
+    def execute(self) -> None:
         callback = self._callback()
         if callback is not None:
             callback(self)
 
-    def preconditions_met(self):
+    def preconditions_met(self) -> bool:
         try:
             client = app.get_client(self._account)
         except Exception:
@@ -212,14 +236,15 @@ class EntityCapsTask(Task):
                 self.entity.jid.bare,
                 groupchat=True)
 
+            assert isinstance(contact, GroupchatContact)
             if not contact.is_joined:
                 self.set_obsolete()
                 return False
 
         return client.state.is_available
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return f'Entity Caps ({self.entity.jid} {self.entity.hash})'
 
-    def __hash__(self):
+    def __hash__(self) -> int:
         return hash(self.entity)
