@@ -1,35 +1,31 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim.  If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 # Chat Markers (XEP-0333)
 
 from __future__ import annotations
 
 from typing import Any
-from typing import Optional
 
+import datetime as dt
+
+import nbxmpp
 from nbxmpp.namespaces import Namespace
-from nbxmpp.structs import StanzaHandler
 from nbxmpp.protocol import JID
+from nbxmpp.structs import DiscoInfo
+from nbxmpp.structs import MessageProperties
+from nbxmpp.structs import StanzaHandler
 
 from gajim.common import app
 from gajim.common import types
 from gajim.common.events import DisplayedReceived
 from gajim.common.events import ReadStateSync
 from gajim.common.modules.base import BaseModule
-from gajim.common.modules.contacts import GroupchatContact
 from gajim.common.modules.contacts import BareContact
+from gajim.common.modules.contacts import GroupchatContact
+from gajim.common.modules.contacts import ResourceContact
+from gajim.common.storage.archive import models as mod
 from gajim.common.structs import OutgoingMessage
 
 
@@ -47,25 +43,37 @@ class ChatMarkers(BaseModule):
                           priority=47),
         ]
 
-    def _process_chat_marker(self,
-                             _client: types.xmppClient,
-                             _stanza: Any,
-                             properties: Any) -> None:
+    def pass_disco(self, info: DiscoInfo) -> None:
+        self.mds_assist_supported = Namespace.MDS_ASSIST in info.features
 
-        if not properties.is_marker or not properties.marker.is_displayed:
+    def _process_chat_marker(self,
+                             _client: types.NBXMPPClient,
+                             _stanza: Any,
+                             properties: MessageProperties) -> None:
+
+        if properties.marker is None or not properties.marker.is_displayed:
             return
 
         if properties.type.is_error:
             return
 
+        self._process(properties)
+        raise nbxmpp.NodeProcessed
+
+    def _process(self, properties: MessageProperties) -> None:
+        assert properties.marker is not None
+        assert properties.jid is not None
+
         jid = properties.jid
         if not properties.is_muc_pm:
             jid = properties.jid.new_as_bare()
 
         if properties.type.is_groupchat:
+            assert properties.muc_jid is not None
             contact = self._client.get_module('Contacts').get_contact(
                 properties.muc_jid,
                 groupchat=True)
+            assert isinstance(contact, GroupchatContact)
             if not contact.is_joined:
                 self._log.warning('Received chat marker while not joined')
                 return
@@ -90,62 +98,74 @@ class ChatMarkers(BaseModule):
                           jid=jid,
                           marker_id=marker_id))
 
-    def _raise_event(self, name: str, properties: Any) -> None:
+    def _raise_event(self, name: str, properties: MessageProperties) -> None:
+        assert properties.marker is not None
+        assert properties.remote_jid is not None
+
         self._log.info('%s: %s %s',
                        name,
                        properties.jid,
                        properties.marker.id)
 
-        jid = properties.jid
         if not properties.is_muc_pm and not properties.type.is_groupchat:
-            jid = properties.jid.bare
+            if properties.mam is not None:
+                timestamp = properties.mam.timestamp
+            else:
+                timestamp = properties.timestamp
 
-        app.storage.archive.set_marker(
-            app.get_jid_from_account(self._account),
-            jid,
-            properties.marker.id,
-            'displayed')
+            timestamp = dt.datetime.fromtimestamp(
+                timestamp, dt.UTC)
+
+            marker_data = mod.DisplayedMarker(
+                account_=self._account,
+                remote_jid_=properties.remote_jid,
+                occupant_=None,
+                id=properties.marker.id,
+                timestamp=timestamp)
+            app.storage.archive.insert_object(marker_data)
 
         app.ged.raise_event(
             DisplayedReceived(account=self._account,
-                              jid=jid,
+                              jid=properties.remote_jid,
                               properties=properties,
                               type=properties.type,
                               is_muc_pm=properties.is_muc_pm,
                               marker_id=properties.marker.id))
 
-    def _send_marker(self,
-                     contact: types.ChatContactT,
-                     marker: str,
-                     id_: str) -> None:
-
-        typ = 'groupchat' if contact.is_groupchat else 'chat'
-
-        message = OutgoingMessage(account=self._account,
-                                  contact=contact,
-                                  message=None,
-                                  type_=typ,
-                                  marker=(marker, id_),
-                                  play_sound=False)
-
-        self._client.send_message(message)
-        self._log.info('Send %s: %s', marker, contact.jid)
-
     def send_displayed_marker(self,
                               contact: types.ChatContactT,
                               message_id: str,
-                              stanza_id: Optional[str]) -> None:
+                              stanza_id: str | None) -> bool:
+
+        # Return value is True if displayed marker was sent and
+        # mds assist was added
 
         if not self._is_sending_marker_allowed(contact):
-            return
+            return False
 
         marker_id = self._determine_marker_id(contact, message_id, stanza_id)
-        self._send_marker(contact, 'displayed', marker_id)
+
+        if not self.mds_assist_supported or isinstance(contact, ResourceContact):
+            # Assist is not added for private groupchat messages see
+            # https://xmpp.org/extensions/xep-0490.html#rules-client
+            stanza_id = None
+
+        message = OutgoingMessage(account=self._account,
+                                  contact=contact,
+                                  marker=('displayed', marker_id),
+                                  mds_id=stanza_id,
+                                  play_sound=False)
+
+        self._client.send_message(message)
+        self._log.info('Send displayed to %s, marker id: %s, mds id: %s',
+                        contact.jid, marker_id, stanza_id)
+
+        return stanza_id is not None
 
     @staticmethod
     def _determine_marker_id(contact: types.ChatContactT,
                              message_id: str,
-                             stanza_id: Optional[str]
+                             stanza_id: str | None
                              ) -> str:
 
         if stanza_id is None:
@@ -158,6 +178,9 @@ class ChatMarkers(BaseModule):
     @staticmethod
     def _is_sending_marker_allowed(contact: types.ChatContactT) -> bool:
         if isinstance(contact, BareContact):
+            if not contact.is_subscribed:
+                return False
+
             return app.settings.get_contact_setting(
                 contact.account, contact.jid, 'send_marker')
 
