@@ -1,16 +1,6 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 # Presence handler
 
@@ -36,6 +26,8 @@ from gajim.common.events import SubscribePresenceReceived
 from gajim.common.events import UnsubscribedPresenceReceived
 from gajim.common.i18n import _
 from gajim.common.modules.base import BaseModule
+from gajim.common.modules.contacts import BareContact
+from gajim.common.modules.contacts import ResourceContact
 from gajim.common.structs import PresenceData
 
 
@@ -86,7 +78,7 @@ class Presence(BaseModule):
         self._jids_for_auto_auth: set[str] = set()
 
     def _presence_received(self,
-                           _con: types.xmppClient,
+                           _con: types.NBXMPPClient,
                            stanza: nbxmpp.protocol.Presence,
                            properties: PresenceProperties
                            ) -> None:
@@ -97,14 +89,16 @@ class Presence(BaseModule):
             # MUC occupant presences are already handled in MUC module
             return
 
+        assert properties.jid is not None
         muc = self._con.get_module('MUC').get_muc_data(properties.jid)
         if muc is not None:
             # Presence from the MUC itself, used for MUC avatar
             # handled in VCardAvatars module
             return
 
-        jid = properties.jid.bare
-        roster_item = self._con.get_module('Roster').get_item(jid)
+        roster_item = self._con.get_module('Roster').get_item(
+            properties.jid.new_as_bare()
+        )
 
         if roster_item is None and not properties.is_self_bare:
             # Handle only presence from roster contacts
@@ -116,9 +110,11 @@ class Presence(BaseModule):
         self._presence_store[properties.jid] = presence_data
 
         contact = self._con.get_module('Contacts').get_contact(properties.jid)
+        assert isinstance(contact, BareContact | ResourceContact)
         contact.update_presence(presence_data)
 
-        if properties.is_self_presence:
+        assert properties.type is not None
+        if properties.is_self_presence and not properties.type.is_unavailable:
             app.ged.raise_event(ShowChanged(account=self._account,
                                             show=properties.show.value))
             return
@@ -153,10 +149,11 @@ class Presence(BaseModule):
         app.ged.raise_event(PresenceReceived(**event_attrs))
 
     def _subscribe_received(self,
-                            _con: types.xmppClient,
+                            _con: types.NBXMPPClient,
                             _stanza: nbxmpp.protocol.Presence,
                             properties: PresenceProperties
                             ) -> None:
+        assert properties.jid is not None
         jid = properties.jid.bare
         fjid = str(properties.jid)
 
@@ -188,10 +185,11 @@ class Presence(BaseModule):
         raise nbxmpp.NodeProcessed
 
     def _subscribed_received(self,
-                             _con: types.xmppClient,
+                             _con: types.NBXMPPClient,
                              _stanza: nbxmpp.protocol.Presence,
                              properties: PresenceProperties
                              ) -> None:
+        assert properties.jid is not None
         jid = properties.jid.bare
         self._log.info('Received Subscribed: %s', properties.jid)
         if jid in self.automatically_added:
@@ -204,7 +202,7 @@ class Presence(BaseModule):
         raise nbxmpp.NodeProcessed
 
     def _unsubscribe_received(self,
-                              _con: types.xmppClient,
+                              _con: types.NBXMPPClient,
                               _stanza: nbxmpp.protocol.Presence,
                               properties: PresenceProperties
                               ) -> None:
@@ -212,10 +210,11 @@ class Presence(BaseModule):
         raise nbxmpp.NodeProcessed
 
     def _unsubscribed_received(self,
-                               _con: types.xmppClient,
+                               _con: types.NBXMPPClient,
                                _stanza: nbxmpp.protocol.Presence,
                                properties: PresenceProperties
                                ) -> None:
+        assert properties.jid is not None
         self._log.info('Received Unsubscribed: %s', properties.jid)
         app.ged.raise_event(UnsubscribedPresenceReceived(
             conn=self._con,
