@@ -168,13 +168,23 @@ class Preview:
     def thumb_exists(self) -> bool:
         if self.thumb_path is None:
             return False
-        return self.thumb_path.exists()
+
+        try:
+            return self.thumb_path.exists()
+        except Exception as error:
+            log.error("Could not check if thumbnail exists: %s", error)
+            return False
 
     @property
     def orig_exists(self) -> bool:
         if self.orig_path is None:
             return False
-        return self.orig_path.exists()
+
+        try:
+            return self.orig_path.exists()
+        except Exception as error:
+            log.error("Could not check if original file exists: %s", error)
+            return False
 
     def update_widget(self, data: bytes | None = None) -> None:
         self._widget.update(self, data)
@@ -351,15 +361,23 @@ class PreviewManager:
                             preview)
 
     def create_thumbnail(self, preview: Preview, data: bytes) -> None:
-        future = app.process_pool.submit(
-            create_thumbnail,
-            data,
-            preview.size,
-            preview.mime_type,
-        )
-        future.add_done_callback(partial(self._write_thumbnail, preview))
+        try:
+            future = app.process_pool.submit(
+                create_thumbnail,
+                data,
+                preview.size,
+                preview.mime_type,
+            )
+            future.add_done_callback(
+                partial(GLib.idle_add, self._write_thumbnail, preview)
+            )
+        except Exception as error:
+            preview.info_message = _('Creating thumbnail failed')
+            preview.update_widget()
+            log.warning('Creating thumbnail failed for: %s %s',
+                        preview.orig_path, error)
 
-    def _write_thumbnail(self, preview: Preview, future: Future[bytes | None]) -> None:
+    def _write_thumbnail(self, preview: Preview, future: Future[bytes | None]) -> bool:
         try:
             result = future.result()
             if result is None:
@@ -369,7 +387,7 @@ class PreviewManager:
             preview.update_widget()
             log.warning('Creating thumbnail failed for: %s %s',
                         preview.orig_path, error)
-            return
+            return GLib.SOURCE_REMOVE
 
         preview.thumbnail = result
 
@@ -379,6 +397,8 @@ class PreviewManager:
                          self._on_thumb_write_finished,
                          preview)
 
+        return GLib.SOURCE_REMOVE
+
     def _process_web_uri(self,
                          uri: str,
                          widget: Any,
