@@ -26,7 +26,6 @@ HAS_ACCOUNT_DEFAULT = _ACCOUNTDEFAULT()
 
 
 BoolSettings = Literal[
-    'allow_open_file_uris',
     'always_english_wikipedia',
     'always_english_wiktionary',
     'ascii_formatting',
@@ -65,7 +64,6 @@ BoolSettings = Literal[
     'preview_verify_https',
     'print_status_in_chats',
     'remote_control',
-    'show_main_menu',
     'send_on_ctrl_enter',
     'show_help_start_chat',
     'show_in_taskbar',
@@ -78,6 +76,7 @@ BoolSettings = Literal[
     'show_subject_on_join',
     'show_trayicon',
     'sort_by_show_in_muc',
+    'sort_by_show_in_start_chat',
     'sounddnd',
     'sounds_on',
     'trayicon_notification_on_events',
@@ -87,6 +86,7 @@ BoolSettings = Literal[
     'use_stun_server',
     'use_urgency_hint',
     'video_see_self',
+    'show_header_bar',
 ]
 
 IntSettings = Literal[
@@ -150,7 +150,6 @@ AllSettingsT = str | int | float | bool | list[str]
 
 APP_SETTINGS: dict[str, str | int | float | bool | list[Any]] = {
     'additional_uri_schemes': '',
-    'allow_open_file_uris': False,
     'always_english_wikipedia': False,
     'always_english_wiktionary': True,
     'app_font_size': 1.0,
@@ -230,9 +229,9 @@ APP_SETTINGS: dict[str, str | int | float | bool | list[Any]] = {
     'roster_theme': 'default',
     'search_engine': 'https://duckduckgo.com/?q=%s',
     'send_on_ctrl_enter': False,
+    'show_header_bar': True,
     'show_help_start_chat': True,
     'show_in_taskbar': True,
-    'show_main_menu': True,
     'show_main_window_on_startup': 'always',
     'show_notifications': True,
     'show_notifications_away': False,
@@ -243,6 +242,7 @@ APP_SETTINGS: dict[str, str | int | float | bool | list[Any]] = {
     'show_subject_on_join': True,
     'show_trayicon': True,
     'sort_by_show_in_muc': False,
+    'sort_by_show_in_start_chat': True,
     'sounddnd': False,
     'sounds_on': True,
     'speller_language': '',
@@ -286,6 +286,7 @@ BoolAccountSettings = Literal[
     'send_marker_default',
     'send_os_info',
     'send_time_info',
+    'sync_muc_blocks',
     'sync_with_global_status',
     'test_ft_proxies_on_startup',
     'use_custom_host',
@@ -441,6 +442,7 @@ ACCOUNT_SETTINGS = {
         'send_os_info': True,
         'send_time_info': True,
         'subscription_request_msg': '',
+        'sync_muc_blocks': True,
         'sync_with_global_status': True,
         'test_ft_proxies_on_startup': False,
         'use_custom_host': False,
@@ -612,7 +614,6 @@ ADVANCED_SETTINGS = {
         'additional_uri_schemes': _(
             'Clickable schemes in addition to the hard-coded list of '
             'IANA-registered ones. Space-separated, lower-case, no colons.'),
-        'allow_open_file_uris': _('Allows opening file:/ URIs when clicked.'),
         'always_english_wikipedia': '',
         'always_english_wiktionary': '',
         'ascii_formatting': _(
