@@ -1,16 +1,6 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from typing import Any
 from typing import cast
@@ -37,8 +27,9 @@ from nbxmpp.protocol import JID
 from gajim import IS_PORTABLE
 from gajim.common import app
 from gajim.common import configpaths
-from gajim.common import optparser
 from gajim.common.setting_values import ACCOUNT_SETTINGS
+from gajim.common.setting_values import AllAccountSettings
+from gajim.common.setting_values import AllAccountSettingsT
 from gajim.common.setting_values import AllContactSettings
 from gajim.common.setting_values import AllContactSettingsT
 from gajim.common.setting_values import AllGroupChatSettings
@@ -74,13 +65,14 @@ from gajim.common.setting_values import WORKSPACE_SETTINGS
 from gajim.common.setting_values import WorkspaceSettings
 from gajim.common.storage.base import Encoder
 from gajim.common.storage.base import json_decoder
+from gajim.common.util.classes import SettingsAction
 
 SETTING_TYPE = bool | int | str | object
 
 
 log = logging.getLogger('gajim.c.settings')
 
-CURRENT_USER_VERSION = 5
+CURRENT_USER_VERSION = 7
 
 CREATE_SQL = '''
     CREATE TABLE settings (
@@ -99,9 +91,10 @@ CREATE_SQL = '''
     INSERT INTO settings(name, settings) VALUES ('proxies', '{proxies}');
     INSERT INTO settings(name, settings) VALUES ('plugins', '{{}}');
     INSERT INTO settings(name, settings) VALUES ('workspaces', '{workspaces}');
+    INSERT INTO settings(name, settings) VALUES ('window_sizes', '{{}}');
 
     PRAGMA user_version={version};
-    '''.format(status=json.dumps(STATUS_PRESET_EXAMPLES),
+    '''.format(status=json.dumps(STATUS_PRESET_EXAMPLES),  # noqa: UP032
                proxies=json.dumps(PROXY_EXAMPLES),
                workspaces=json.dumps(INITAL_WORKSPACE),
                version=CURRENT_USER_VERSION)
@@ -125,6 +118,7 @@ class SettingsDictT(TypedDict):
     soundevents: dict[str, dict[str, Any]]
     status_presets: dict[str, dict[str, str]]
     proxies: dict[str, dict[str, Any]]
+    window_sizes: dict[str, tuple[int, int]]
 
 
 class Settings:
@@ -157,10 +151,25 @@ class Settings:
         for handlers in self._callbacks.values():
             for handler in list(handlers):
                 if isinstance(handler, tuple):
-                    continue
-                func = handler()
-                if func is None or func.__self__ is object_:
-                    handlers.remove(handler)
+                    widget = handler[1]
+                    if widget is object_:
+                        handlers.remove(handler)
+                else:
+                    func = handler()
+                    if func is None or func.__self__ is object_:
+                        handlers.remove(handler)
+
+    def create_action(self, setting: BoolSettings) -> SettingsAction:
+        # Currently only boolean app settings are supported
+        value = self.get_app_setting(setting)
+        action = SettingsAction(
+            name=setting.replace("_", "-"),
+            parameter_type=GLib.VariantType("b"),
+            state=GLib.Variant("b", value),
+        )
+
+        self.bind_signal(setting, action, "change_state")
+        return action
 
     def bind_signal(self,
                     setting: str,
@@ -174,12 +183,7 @@ class Settings:
 
         callbacks = self._callbacks[(setting, account, jid)]
         func = getattr(widget, func_name)
-        callbacks.append((func, inverted, default_text))
-
-        def _on_destroy(*args: Any) -> None:
-            callbacks.remove((func, inverted, default_text))
-
-        widget.connect('destroy', _on_destroy)
+        callbacks.append((func, widget, inverted, default_text))
 
     def _notify(self,
                 value: Any,
@@ -192,7 +196,7 @@ class Settings:
         callbacks = self._callbacks[(setting, account, jid)]
         for func in list(callbacks):
             if isinstance(func, tuple):
-                func, inverted, default_text = func
+                func, widget, inverted, default_text = func
                 if isinstance(value, bool) and inverted:
                     value = not value
 
@@ -226,11 +230,6 @@ class Settings:
             self._connect_database()
         self._load_settings()
         self._load_account_settings()
-        if not self._settings['app']:
-            # ['app'] is empty in a newly created database.
-            # If there is an old config, it gets migrated at this point.
-            self._migrate_old_config()
-            self._commit()
         self._migrate_database()
         self._load_app_overrides()
         self._commit()
@@ -350,6 +349,7 @@ class Settings:
                      VALUES ('workspaces', ?)'''
             self._con.execute(sql, (json.dumps(INITAL_WORKSPACE),))
             self._settings['workspaces'] = INITAL_WORKSPACE
+            self._commit_settings('workspaces')
             self._set_user_version(1)
 
         if version < 2:
@@ -367,6 +367,7 @@ class Settings:
                 workspace['chats'] = open_chats
                 workspace.pop('open_chats', None)
 
+            self._commit_settings('workspaces')
             self._set_user_version(2)
 
         if version < 3:
@@ -375,6 +376,9 @@ class Settings:
                 if account_settings['account'].get('active') is None:
                     account_settings['account']['active'] = True
 
+            for account in self._account_settings:
+                self._commit_account_settings(account)
+
             self._set_user_version(3)
 
         if version < 4:
@@ -386,196 +390,38 @@ class Settings:
             if value is not None:
                 self._settings['app']['date_format'] = value
 
+            self._commit_settings('app')
             self._set_user_version(4)
 
         if version < 5:
             self._settings['app'].pop('muclumbus_api_http_uri', None)
+            self._commit_settings('app')
             self._set_user_version(5)
 
-    def _migrate_old_config(self) -> None:
-        if self._in_memory:
-            return
+        if version < 6:
+            for account_settings in self._account_settings.values():
+                settings = account_settings['account']
+                localpart = settings.get('name', '')
+                domain = settings.get('hostname', '')
+                try:
+                    address = JID.from_string(f'{localpart}@{domain}')
+                except Exception as error:
+                    log.warning('Unable to migrate address: %s', error)
+                    address = JID.from_string('unknown@unknown.invalid')
 
-        config_file = configpaths.get('CONFIG_FILE')
-        if not config_file.exists():
-            return
+                settings['address'] = str(address)
 
-        # Read legacy config
-        optparser.OptionsParser(str(configpaths.get('CONFIG_FILE'))).read()
+            for account in self._account_settings:
+                self._commit_account_settings(account)
 
-        account_settings = app.config.get_all_per('accounts')
-        self._cleanup_account_default_values('account', account_settings)
+            self._set_user_version(6)
 
-        contact_settings = app.config.get_all_per('contacts')
-        self._cleanup_account_default_values('contact', contact_settings)
-
-        group_chat_settings = app.config.get_all_per('rooms')
-        self._cleanup_account_default_values('group_chat',
-                                             group_chat_settings)
-
-        for account, settings in account_settings.items():
-            if account == 'Local':
-                # Zeroconf support was dropped so don’t migrate the account
-                continue
-            self.add_account(account)
-            self._account_settings[account]['account'] = settings
-            self._account_settings[account]['contact'] = contact_settings
-            self._account_settings[account]['group_chat'] = group_chat_settings
-            self._commit_account_settings(account)
-
-        self._migrate_encryption_settings()
-
-        # Migrate plugin settings
-        self._settings['plugins'] = app.config.get_all_per('plugins')
-        self._commit_settings('plugins')
-
-        self._migrate_app_settings()
-        self._migrate_soundevent_settings()
-        self._migrate_status_preset_settings()
-        self._migrate_proxy_settings()
-
-        new_path = config_file.with_name(f'{config_file.name}.old')
-        config_file.rename(new_path)
-        log.info('Successfully migrated config')
-
-    def _migrate_app_settings(self) -> None:
-        app_settings = app.config.get_all()
-
-        # Migrate deprecated settings
-        value = app_settings.pop('send_chatstate_muc_default', None)
-        if value is not None:
-            for settings in self._account_settings.values():
-                settings['account']['gc_send_chatstate_default'] = value
-
-        value = app_settings.pop('send_chatstate_default', None)
-        if value is not None:
-            for settings in self._account_settings.values():
-                settings['account']['send_chatstate_default'] = value
-
-        value = app_settings.pop('print_join_left_default', None)
-        if value is not None:
-            app_settings['gc_print_join_left_default'] = value
-
-        value = app_settings.pop('print_status_muc_default', None)
-        if value is not None:
-            app_settings['gc_print_status_default'] = value
-
-        # Cleanup values which are equal to current defaults
-        for setting, value in list(app_settings.items()):
-            if (setting not in APP_SETTINGS or
-                    value == APP_SETTINGS[setting]):
-                del app_settings[setting]
-
-        self._settings['app'] = app_settings
-        self._commit_settings('app')
-
-        for account in self._account_settings:
-            self._commit_account_settings(account)
-
-    def _migrate_encryption_settings(self) -> None:
-        # Migrate encryption settings into contact/group chat settings
-        encryption_settings = app.config.get_all_per('encryption')
-        for key, settings in encryption_settings.items():
-            account, jid = self._split_encryption_config_key(key)
-            if account is None:
-                continue
-
-            assert jid is not None
-            encryption = settings.get('encryption')
-            if not encryption:
-                continue
-
-            if '@' not in jid:
-                continue
-
-            # Sad try to determine if the jid is a group chat
-            # At this point there is no better way
-            domain = jid.split('@')[1]
-            subdomain = domain.split('.')[0]
-            if subdomain in ('muc', 'conference', 'conf',
-                             'rooms', 'room', 'chat'):
-                category = 'group_chat'
-            else:
-                category = 'contact'
-
-            if jid not in self._account_settings[account][category]:
-                self._account_settings[account][category][jid] = {
-                    'encryption': encryption}
-            else:
-                self._account_settings[account][category][
-                    jid]['encryption'] = encryption
-            self._commit_account_settings(account)
-
-    def _split_encryption_config_key(self, key: str) -> tuple[str | None,
-                                                              str | None]:
-        for account in self._account_settings:
-            if not key.startswith(account):
-                continue
-            jid = key.replace(f'{account}-', '', 1)
-            return account, jid
-        return None, None
-
-    def _migrate_soundevent_settings(self) -> None:
-        soundevent_settings = app.config.get_all_per('soundevents')
-        for soundevent, settings in list(soundevent_settings.items()):
-            if soundevent not in DEFAULT_SOUNDEVENT_SETTINGS:
-                del soundevent_settings[soundevent]
-                continue
-
-            for setting, value in list(settings.items()):
-                if DEFAULT_SOUNDEVENT_SETTINGS[soundevent][setting] == value:
-                    del soundevent_settings[soundevent][setting]
-                    if not soundevent_settings[soundevent]:
-                        del soundevent_settings[soundevent]
-
-        self._settings['soundevents'] = soundevent_settings
-        self._commit_settings('soundevents')
-
-    def _migrate_status_preset_settings(self) -> None:
-        status_preset_settings = app.config.get_all_per('statusmsg')
-        for preset, settings in list(status_preset_settings.items()):
-            if '_last_' in preset:
-                del status_preset_settings[preset]
-                continue
-
-            for setting, value in list(settings.items()):
-                if setting not in STATUS_PRESET_SETTINGS:
-                    continue
-                if STATUS_PRESET_SETTINGS[setting] == value:
-                    del status_preset_settings[preset][setting]
-                    if not status_preset_settings[preset]:
-                        del status_preset_settings[preset]
-
-        self._settings['status_presets'] = status_preset_settings
-        self._commit_settings('status_presets')
-
-    def _migrate_proxy_settings(self) -> None:
-        proxy_settings = app.config.get_all_per('proxies')
-        for proxy_name, settings in proxy_settings.items():
-            for setting, value in list(settings.items()):
-                if (setting not in PROXY_SETTINGS or
-                        PROXY_SETTINGS[setting] == value):
-                    del proxy_settings[proxy_name][setting]
-
-        self._settings['proxies'] = proxy_settings
-        self._commit_settings('proxies')
-
-    @staticmethod
-    def _cleanup_account_default_values(category: str, settings: Any) -> None:
-        for contact, settings_ in list(settings.items()):
-            for setting, value in list(settings_.items()):
-                if setting not in ACCOUNT_SETTINGS[category]:
-                    del settings[contact][setting]
-                    if not settings[contact]:
-                        del settings[contact]
-                    continue
-
-                default = ACCOUNT_SETTINGS[category][setting]
-                if default == value:
-                    del settings[contact][setting]
-                    if not settings[contact]:
-                        del settings[contact]
-                    continue
+        if version < 7:
+            sql = '''INSERT INTO settings(name, settings)
+                     VALUES ('window_sizes', '{}')'''
+            self._con.execute(sql)
+            self._settings['window_sizes'] = {}
+            self._set_user_version(7)
 
     def close(self) -> None:
         log.info('Close settings')
@@ -711,6 +557,13 @@ class Settings:
 
     set = set_app_setting
 
+    def set_window_size(self, window_name: str, width: int, height: int) -> None:
+        self._settings['window_sizes'][window_name] = (width, height)
+        self._commit_settings('window_sizes')
+
+    def get_window_size(self, window_name: str) -> tuple[int, int] | None:
+        return self._settings['window_sizes'].get(window_name)
+
     def get_plugin_setting(self, plugin: str, setting: str) -> SETTING_TYPE:
         if setting not in PLUGIN_SETTINGS:
             raise ValueError(f'Invalid plugin setting: {setting}')
@@ -754,6 +607,9 @@ class Settings:
 
     def add_account(self, account: str) -> None:
         log.info('Add account: %s', account)
+        if account in self._account_settings:
+            raise ValueError('Account %s exists already' % account)
+
         self._account_settings[account] = {'account': {},
                                            'contact': {},
                                            'group_chat': {}}
@@ -775,6 +631,13 @@ class Settings:
     def get_accounts(self) -> list[str]:
         return list(self._account_settings.keys())
 
+    def account_exists(self, jid: str) -> bool:
+        for account in self._account_settings:
+            address = self.get_account_setting(account, 'address')
+            if jid == JID.from_string(address):
+                return True
+        return False
+
     def get_active_accounts(self) -> list[str]:
         active: list[str] = []
         for account in self._account_settings:
@@ -782,6 +645,14 @@ class Settings:
                 active.append(account)
         return active
 
+    def get_account_from_jid(self, jid: JID) -> str:
+        for account in self._account_settings:
+            address = self.get_account_setting(account, 'address')
+            if jid == JID.from_string(address):
+                return account
+
+        raise ValueError(f'No account found for: {jid}')
+
     @overload
     def get_account_setting(self,
                             account: str,
@@ -802,7 +673,7 @@ class Settings:
 
     def get_account_setting(self,
                             account: str,
-                            setting: str) -> AllSettingsT:
+                            setting: AllAccountSettings) -> AllAccountSettingsT:
 
         if account not in self._account_settings:
             raise ValueError(f'Account missing: {account}')
@@ -838,8 +709,8 @@ class Settings:
 
     def set_account_setting(self,
                             account: str,
-                            setting: str,
-                            value: AllSettingsT | None) -> None:
+                            setting: AllAccountSettings,
+                            value: AllAccountSettingsT | None) -> None:
 
         if account not in self._account_settings:
             raise ValueError(f'Account missing: {account}')
@@ -924,12 +795,18 @@ class Settings:
                 return self.get_app_setting(f'gc_{setting}_default')
 
             if default is HAS_ACCOUNT_DEFAULT:
-                context_default_setting = f'gc_{setting}_{context}_default'
-                if context_default_setting in ACCOUNT_SETTINGS['account']:
-                    return self.get_account_setting(account,
-                                                    context_default_setting)
-                return self.get_account_setting(account,
-                                                f'gc_{setting}_default')
+                default_settings = [
+                    f'gc_{setting}_{context}_default',
+                    f'gc_{setting}_default',
+                    f'{setting}_default',
+                ]
+
+                for default_setting in default_settings:
+                    if default_setting in ACCOUNT_SETTINGS['account']:
+                        return self.get_account_setting(
+                            account, default_setting)
+
+                raise ValueError(f'No default setting found for {setting}')
 
             return default
 
@@ -986,9 +863,20 @@ class Settings:
             if default is HAS_ACCOUNT_DEFAULT:
                 default_store = ACCOUNT_SETTINGS['account']
 
-            default = default_store.get(f'gc_{setting}_{context}_default')
+            default_settings = [
+                f'gc_{setting}_{context}_default',
+                f'gc_{setting}_default',
+                f'{setting}_default',
+            ]
+
+            default = None
+            for default_setting in default_settings:
+                if default_setting in default_store:
+                    default = default_store.get(default_setting)
+                    break
+
             if default is None:
-                default = default_store.get(f'gc_{setting}_default')
+                raise ValueError(f'No default setting found for {setting}')
 
         if not isinstance(value, type(default)) and value is not None:
             raise TypeError(f'Invalid type for {setting}: '
@@ -1367,29 +1255,3 @@ class Settings:
         self._commit()
         self._con.close()
         del self._con
-
-
-class LegacyConfig:
-
-    @staticmethod
-    def get(setting: str) -> SETTING_TYPE:
-        return app.settings.get_app_setting(setting)
-
-    @staticmethod
-    def set(setting: str, value: SETTING_TYPE) -> None:
-        app.settings.set_app_setting(setting, value)
-
-    @staticmethod
-    def get_per(kind: str, key: str, setting: str) -> SETTING_TYPE:
-        if kind == 'accounts':
-            return app.settings.get_account_setting(key, setting)
-
-        if kind == 'plugins':
-            return app.settings.get_plugin_setting(key, setting)
-        raise ValueError
-
-    @staticmethod
-    def set_per(kind: str, key: str, setting: str, value: SETTING_TYPE) -> None:
-        if kind == 'accounts':
-            app.settings.set_account_setting(key, setting, value)
-        raise ValueError
