@@ -29,6 +29,7 @@ from sqlalchemy import update
 from sqlalchemy.dialects.sqlite import insert
 from sqlalchemy.engine import Engine
 from sqlalchemy.exc import IntegrityError
+from sqlalchemy.orm import joinedload
 from sqlalchemy.orm import Session
 
 from gajim.common import app
@@ -48,6 +49,7 @@ from gajim.common.storage.archive.models import Moderation
 from gajim.common.storage.archive.models import Occupant
 from gajim.common.storage.archive.models import Reaction
 from gajim.common.storage.archive.models import Remote
+from gajim.common.storage.archive.models import Retraction
 from gajim.common.storage.archive.models import Thread
 from gajim.common.storage.base import AlchemyStorage
 from gajim.common.storage.base import timeit
@@ -57,7 +59,7 @@ from gajim.common.storage.base import with_session_yield_from
 from gajim.common.util.datetime import FIRST_UTC_DATETIME
 from gajim.common.util.text import get_random_string
 
-CURRENT_USER_VERSION = 11
+CURRENT_USER_VERSION = 13
 
 
 log = logging.getLogger('gajim.c.storage.archive')
@@ -208,6 +210,8 @@ class MessageArchiveStorage(AlchemyStorage):
     def insert_object(
         self, session: Session, obj: Any, ignore_on_conflict: bool = True
     ) -> int:
+
+        obj.validate()
         self._set_foreign_keys(session, obj)
         self._log_row(obj)
         session.add(obj)
@@ -247,6 +251,8 @@ class MessageArchiveStorage(AlchemyStorage):
         return_pk_on_conflict: bool = False,
         ignore_on_conflict: bool = False,
     ) -> int:
+
+        row.validate()
         self._set_foreign_keys(session, row)
         self._log_row(row)
         table = row.__class__
@@ -277,6 +283,8 @@ class MessageArchiveStorage(AlchemyStorage):
         session: Session,
         row: Any,
     ) -> int:
+
+        row.validate()
         self._set_foreign_keys(session, row)
         self._log_row(row)
         table = row.__class__
@@ -312,6 +320,8 @@ class MessageArchiveStorage(AlchemyStorage):
         session: Session,
         row: Any,
     ) -> int | None:
+
+        row.validate()
         self._set_foreign_keys(session, row)
         self._log_row(row)
         table = row.__class__
@@ -414,6 +424,9 @@ class MessageArchiveStorage(AlchemyStorage):
         if message.moderation is not None:
             session.delete(message.moderation)
 
+        if message.retraction is not None:
+            session.delete(message.retraction)
+
         session.delete(message)
 
     @with_session
@@ -740,7 +753,8 @@ class MessageArchiveStorage(AlchemyStorage):
             Message.fk_remote_pk == fk_remote_pk,
             Message.fk_account_pk == fk_account_pk,
             Message.fk_occupant_pk == fk_occupant_pk,
-            Message.stanza_id.isnot(None)
+            Message.stanza_id.isnot(None),
+            ~Message.moderation.has(),
         )
         result = cast(Sequence[str] | None, session.scalars(stmt).all())
         return result
@@ -810,6 +824,7 @@ class MessageArchiveStorage(AlchemyStorage):
                 Message.text.ilike(f'%{query}%'),
                 Message.timestamp.between(after, before),
                 ~Message.moderation.has(),
+                ~Message.retraction.has(),
             )
             .order_by(sa.desc(Message.timestamp), sa.desc(Message.pk))
             .execution_options(yield_per=25)
@@ -1049,6 +1064,8 @@ class MessageArchiveStorage(AlchemyStorage):
         fk_account_pk = self._get_account_pk(session, account)
         fk_remote_pk = self._get_jid_pk(session, jid)
 
+        # Errors
+
         stmt = delete(MessageError).where(
             MessageError.fk_account_pk == fk_account_pk,
             MessageError.fk_remote_pk == fk_remote_pk,
@@ -1056,6 +1073,8 @@ class MessageArchiveStorage(AlchemyStorage):
 
         session.execute(stmt)
 
+        # Moderation
+
         stmt = delete(Moderation).where(
             Moderation.fk_account_pk == fk_account_pk,
             Moderation.fk_remote_pk == fk_remote_pk,
@@ -1063,6 +1082,17 @@ class MessageArchiveStorage(AlchemyStorage):
 
         session.execute(stmt)
 
+        # Retraction
+
+        stmt = delete(Retraction).where(
+            Retraction.fk_account_pk == fk_account_pk,
+            Retraction.fk_remote_pk == fk_remote_pk,
+        )
+
+        session.execute(stmt)
+
+        # Messages
+
         stmt = delete(Message).where(
             Message.fk_account_pk == fk_account_pk, Message.fk_remote_pk == fk_remote_pk
         )
@@ -1080,6 +1110,7 @@ class MessageArchiveStorage(AlchemyStorage):
 
         session.execute(delete(MessageError))
         session.execute(delete(Moderation))
+        session.execute(delete(Retraction))
         session.execute(delete(Message))
 
         log.info('Removed all chat history')
@@ -1132,6 +1163,8 @@ class MessageArchiveStorage(AlchemyStorage):
                 Message.fk_account_pk == fk_account_pk,
                 Message.fk_remote_pk == fk_remote_pk,
                 Message.correction_id.is_(None),
+                ~Message.moderation.has(),
+                ~Message.retraction.has(),
             )
             .order_by(Message.timestamp, Message.pk)
             .execution_options(yield_per=25)
@@ -1139,3 +1172,53 @@ class MessageArchiveStorage(AlchemyStorage):
 
         self._explain(session, stmt)
         yield from session.scalars(stmt)
+
+    @with_session
+    @timeit
+    def get_blocked_occupants(
+        self, session: Session, account: str, jid: JID | None = None,
+    ) -> Sequence[Occupant]:
+        fk_account_pk = self._get_account_pk(session, account)
+
+        stmt = select(Occupant)
+
+        if jid is not None:
+            fk_remote_pk = self._get_jid_pk(session, jid)
+            stmt = stmt.where(Occupant.fk_remote_pk == fk_remote_pk)
+
+        stmt = stmt.where(
+            Occupant.fk_account_pk == fk_account_pk,
+            Occupant.blocked == True,  # noqa: E712
+        ).options(joinedload(Occupant.remote))
+
+        self._explain(session, stmt)
+        return session.scalars(stmt).all()
+
+    @with_session
+    @timeit
+    def set_block_occupant(
+        self,
+        session: Session,
+        account: str,
+        jid: JID | None,
+        occupant_ids: Sequence[str],
+        value: bool,
+    ) -> int:
+        fk_account_pk = self._get_account_pk(session, account)
+
+        stmt = update(Occupant)
+        if occupant_ids:
+            stmt = stmt.where(Occupant.id.in_(occupant_ids))
+
+        if jid is not None:
+            fk_remote_pk = self._get_jid_pk(session, jid)
+            stmt = stmt.where(Occupant.fk_remote_pk == fk_remote_pk)
+
+        stmt = stmt.where(
+            Occupant.fk_account_pk == fk_account_pk,
+            Occupant.blocked != value,
+
+        ).values(blocked=value)
+
+        self._explain(session, stmt)
+        return session.execute(stmt).rowcount
