@@ -11,12 +11,11 @@ import locale
 import logging
 from collections import defaultdict
 
+from gi.repository import Adw
 from gi.repository import Gdk
-from gi.repository import GLib
 from gi.repository import GObject
 from gi.repository import Gtk
 from gi.repository import Pango
-from nbxmpp.stringprep import saslprep
 
 from gajim.common import app
 from gajim.common import passwords
@@ -26,17 +25,18 @@ from gajim.common.i18n import _
 from gajim.common.i18n import p_
 from gajim.common.setting_values import AllSettingsT
 
+from gajim.gtk.alert import AlertDialog
+from gajim.gtk.alert import CancelDialogResponse
+from gajim.gtk.alert import DialogResponse
 from gajim.gtk.const import Setting
 from gajim.gtk.const import SettingKind
 from gajim.gtk.const import SettingType
-from gajim.gtk.dialogs import ConfirmationDialog
-from gajim.gtk.dialogs import DialogButton
 from gajim.gtk.filechoosers import Filter
-from gajim.gtk.menus import build_accounts_menu
 from gajim.gtk.omemo_trust_manager import OMEMOTrustManager
 from gajim.gtk.settings import DropDownSetting
 from gajim.gtk.settings import SettingsBox
 from gajim.gtk.settings import SettingsDialog
+from gajim.gtk.structs import ExportHistoryParam
 from gajim.gtk.util.classes import SignalManager
 from gajim.gtk.util.misc import iterate_listbox_children
 from gajim.gtk.util.window import get_app_window
@@ -109,18 +109,23 @@ class AccountsWindow(GajimAppWindow):
 
     def _relog(self, account: str) -> None:
 
-        def relog():
-            client = app.get_client(account)
-            client.disconnect(gracefully=True, reconnect=True, destroy_client=True)
+        def _on_response(response_id: str) -> None:
+            if response_id == "accept":
+                client = app.get_client(account)
+                client.disconnect(gracefully=True, reconnect=True, destroy_client=True)
+
             self.close()
 
-        ConfirmationDialog(
+        AlertDialog(
             _("Re-Login Now?"),
             _("To apply all changes instantly, you have to re-login."),
-            [
-                DialogButton.make("Cancel", text=_("_Later"), callback=self.close),
-                DialogButton.make("Accept", text=_("_Re-Login"), callback=relog),
+            responses=[
+                DialogResponse("cancel", _("_Later")),
+                DialogResponse(
+                    "accept", _("_Re-Login"), is_default=True, appearance="suggested"
+                ),
             ],
+            callback=_on_response,
         )
 
     @staticmethod
@@ -358,7 +363,7 @@ class RemoveMenuItem(MenuItem):
         image = Gtk.Image.new_from_icon_name("user-trash-symbolic")
 
         self.set_selectable(False)
-        image.add_css_class("error-color")
+        image.add_css_class("error")
 
         self._box.append(image)
         self._box.append(self._label)
@@ -374,7 +379,6 @@ class AccountLabelMenuItem(MenuItem):
         self.set_activatable(False)
 
         image = Gtk.Image.new_from_icon_name("avatar-default-symbolic")
-        image.add_css_class("insensitive-fg-color")
 
         self._label.add_css_class("accounts-label-row")
         self._label.set_ellipsize(Pango.EllipsizeMode.END)
@@ -397,8 +401,7 @@ class BackMenuItem(MenuItem):
 
         self._label.set_text(_("Back"))
 
-        image = Gtk.Image.new_from_icon_name("go-previous-symbolic")
-        image.add_css_class("insensitive-fg-color")
+        image = Gtk.Image.new_from_icon_name("lucide-chevron-left-symbolic")
 
         self._box.append(image)
         self._box.append(self._label)
@@ -417,7 +420,7 @@ class PageMenuItem(MenuItem):
         elif name == "connection":
             icon = "feather-globe-symbolic"
         elif name == "advanced":
-            icon = "preferences-system-symbolic"
+            icon = "lucide-settings-symbolic"
         else:
             icon = "dialog-error-symbolic"
 
@@ -491,8 +494,7 @@ class AccountRow(Gtk.ListBoxRow, SignalManager):
         self._label.set_xalign(0)
         self._label.set_width_chars(18)
 
-        next_icon = Gtk.Image.new_from_icon_name("go-next-symbolic")
-        next_icon.add_css_class("insensitive-fg-color")
+        next_icon = Gtk.Image.new_from_icon_name("lucide-chevron-right-symbolic")
 
         account_enabled = app.settings.get_account_setting(self._account, "active")
         self._switch = Gtk.Switch()
@@ -541,12 +543,15 @@ class AccountRow(Gtk.ListBoxRow, SignalManager):
 
     def _on_enable_switch(self, switch: Gtk.Switch, state: bool, account: str) -> int:
 
-        def _disable() -> None:
-            client = app.get_client(account)
-            client.connect_signal("state-changed", self._on_state_changed)
-            client.change_status("offline", "offline")
-            switch.set_state(state)
-            self._set_label(state)
+        def _on_response(response_id: str) -> None:
+            if response_id == "disable":
+                client = app.get_client(account)
+                client.connect_signal("state-changed", self._on_state_changed)
+                client.change_status("offline", "offline")
+                switch.set_state(state)
+                self._set_label(state)
+            else:
+                switch.set_active(True)
 
         account_is_active = app.settings.get_account_setting(account, "active")
         if account_is_active == state:
@@ -558,22 +563,21 @@ class AccountRow(Gtk.ListBoxRow, SignalManager):
             assert window is not None
 
             account_label = app.get_account_label(account)
-            ConfirmationDialog(
+            AlertDialog(
                 _("Disable Account?"),
                 _(
                     "Account %(name)s is still connected\n"
                     "All chat and group chat windows will be closed."
                 )
                 % {"name": account_label},
-                [
-                    DialogButton.make(
-                        "Cancel", callback=lambda: switch.set_active(True)
-                    ),
-                    DialogButton.make(
-                        "Remove", text=_("_Disable Account"), callback=_disable
+                responses=[
+                    CancelDialogResponse(),
+                    DialogResponse(
+                        "disable", _("_Disable Account"), appearance="destructive"
                     ),
                 ],
-                transient_for=window.window,
+                callback=_on_response,
+                parent=window.window,
             )
             return Gdk.EVENT_STOP
 
@@ -615,14 +619,13 @@ class GenericSettingPage(Gtk.Box):
     name = ""
 
     def __init__(self, account: str, settings: list[Setting]) -> None:
-        Gtk.Box.__init__(self, orientation=Gtk.Orientation.VERTICAL, spacing=12)
+        Gtk.Box.__init__(self, orientation=Gtk.Orientation.VERTICAL)
         self.set_valign(Gtk.Align.START)
         self.set_vexpand(True)
-        self.add_css_class("settings-page")
         self.account = account
 
         self.listbox = SettingsBox(account)
-        self.listbox.add_css_class("border")
+        self.listbox.add_css_class("mt-18")
         self.listbox.set_selection_mode(Gtk.SelectionMode.NONE)
         self.listbox.set_vexpand(False)
         self.listbox.set_valign(Gtk.Align.END)
@@ -631,7 +634,8 @@ class GenericSettingPage(Gtk.Box):
             self.listbox.add_setting(setting)
         self.listbox.update_states()
 
-        self.append(self.listbox)
+        clamp = Adw.Clamp(child=self.listbox)
+        self.append(clamp)
 
     def connect_signal(self, stack: Gtk.Stack) -> int:
         return stack.connect("notify::visible-child", self._on_visible_child_changed)
@@ -728,7 +732,6 @@ class GeneralPage(GenericSettingPage):
         window = get_app_window("AccountsWindow")
         assert window is not None
         window.update_account_label(self.account)
-        build_accounts_menu()
 
 
 class PrivacyPage(GenericSettingPage):
@@ -765,10 +768,7 @@ class PrivacyPage(GenericSettingPage):
             "PGP": "PGP",
         }
 
-        export_history_variant = GLib.Variant(
-            "a{sv}",
-            {"account": GLib.Variant("s", account), "jid": GLib.Variant("s", "")},
-        )
+        param = ExportHistoryParam(account=account, jid=None)
 
         settings = [
             Setting(
@@ -878,6 +878,15 @@ class PrivacyPage(GenericSettingPage):
                     "button-callback": self._reset_send_read_marker,
                 },
             ),
+            Setting(
+                SettingKind.SWITCH,
+                _("Sync Group Chat Blocklist"),
+                SettingType.ACCOUNT_CONFIG,
+                "sync_muc_blocks",
+                callback=self._sync_blocks,
+                enabled_func=self._get_sync_blocks_enabled,
+                desc=_("Sync group chat blocklist with other devices"),
+            ),
             Setting(
                 SettingKind.DROPDOWN,
                 _("Keep Chat History"),
@@ -891,7 +900,7 @@ class PrivacyPage(GenericSettingPage):
                 _("Export Chat History"),
                 SettingType.ACTION,
                 "app.export-history",
-                props={"variant": export_history_variant},
+                props={"variant": param.to_variant()},
                 desc=_("Export your chat history from Gajim"),
             ),
         ]
@@ -934,6 +943,19 @@ class PrivacyPage(GenericSettingPage):
         app.settings.set_contact_settings("send_marker", None)
         app.settings.set_group_chat_settings("send_marker", None, context="private")
 
+    def _get_sync_blocks_enabled(self) -> bool:
+        if self._client is None:
+            return False
+
+        if not self._client.state.is_available:
+            return False
+
+        return self._client.get_module("Bookmarks").nativ_bookmarks_used
+
+    def _sync_blocks(self, state: bool, _data: Any) -> None:
+        if self._client is not None and state:
+            self._client.get_module("MucBlocking").merge_blocks()
+
 
 class EncryptionOMEMOPage(GenericSettingPage):
 
@@ -951,21 +973,20 @@ class EncryptionOMEMOPage(GenericSettingPage):
         ]
         GenericSettingPage.__init__(self, account, settings)
 
-        btbv_label = Gtk.Label()
-        btbv_label.set_xalign(0)
+        title_heading = _("Trust Management")
         wiki_url = "https://dev.gajim.org/gajim/gajim/-/wikis/help/OMEMO"
         link_text = _("Read more about blind trust")
-        markup = f'<a href="{wiki_url}">{link_text}</a>'
-        btbv_label.set_markup(markup)
-        self.prepend(btbv_label)
+        link_markup = f'<a href="{wiki_url}">{link_text}</a>'
 
-        heading = Gtk.Label(label=_("Trust Management"))
-        heading.add_css_class("bold")
-        heading.set_xalign(0)
-        self.prepend(heading)
+        preferences_group = Adw.PreferencesGroup(
+            title=f"{title_heading}\n{link_markup}"
+        )
+        preferences_group.add_css_class("mt-18")
+        clamp = Adw.Clamp(child=preferences_group)
+        self.prepend(clamp)
 
         omemo_trust_manager = OMEMOTrustManager(account)
-        omemo_trust_manager.set_margin_top(18)
+        omemo_trust_manager.add_css_class("mt-18")
         self.append(omemo_trust_manager)
 
 
@@ -984,7 +1005,7 @@ class ConnectionPage(GenericSettingPage):
                 name="proxy",
                 props={
                     "data": self._get_proxies(),
-                    "button-icon-name": "preferences-system-symbolic",
+                    "button-icon-name": "lucide-settings-symbolic",
                     "button-callback": self._on_proxy_edit,
                 },
             ),
@@ -1251,12 +1272,6 @@ class LoginDialog(SettingsDialog):
         )
 
     def on_password_change(self, new_password: str, _data: Any) -> None:
-        try:
-            new_password = saslprep(new_password)
-        except Exception:
-            # TODO: Show warning for invalid passwords
-            return
-
         passwords.save_password(self.account, new_password)
 
     def _cleanup(self) -> None:
