@@ -356,9 +356,9 @@ class ChatList(Gtk.ListBox, EventHelper, SignalManager):
 
     def _emit_unread_changed(self) -> None:
         count = self.get_unread_count()
-        chat_list = self.get_parent()
-        assert chat_list is not None
-        chat_list.emit("unread-count-changed", self._workspace_id, count)
+        chat_list_stack = self.get_parent()
+        assert chat_list_stack is not None
+        chat_list_stack.emit("unread-count-changed", self._workspace_id, count)
 
     def _get_row_before(self, row: ChatListRow) -> ChatListRow | None:
         row_before = self.get_row_at_index(row.get_index() - 1)
