@@ -39,6 +39,7 @@ from gajim.common.util.preview import format_geo_coords
 from gajim.common.util.preview import guess_simple_file_type
 from gajim.common.util.preview import split_geo_uri
 from gajim.common.util.user_strings import get_moderation_text
+from gajim.common.util.user_strings import get_retraction_text
 from gajim.common.util.user_strings import get_uf_relative_time
 
 from gajim.gtk.builder import get_builder
@@ -101,6 +102,7 @@ class ChatListRow(Gtk.ListBoxRow, SignalManager):
         self._connect(self._ui.close_button, "clicked", self._on_close_button_clicked)
 
         self._menu_popover = GajimPopover(None)
+        self._connect(self._menu_popover, "closed", self._on_context_popover_closed)
         self._ui.mainbox.append(self._menu_popover)
 
         self._connect(self, "state-flags-changed", self._on_state_flags_changed)
@@ -121,9 +123,6 @@ class ChatListRow(Gtk.ListBoxRow, SignalManager):
         )
         self._on_mute_setting_changed()
 
-        self._drag_hotspot_x: float = 0
-        self._drag_hotspot_y: float = 0
-
         drag_source = Gtk.DragSource(actions=Gdk.DragAction.MOVE)
         self._connect(drag_source, "prepare", self._on_prepare)
         self._connect(drag_source, "drag-begin", self._on_drag_begin)
@@ -159,15 +158,18 @@ class ChatListRow(Gtk.ListBoxRow, SignalManager):
         if message.text is not None:
             message_text = message.text
 
-            if message.corrections:
-                message_text = message.get_last_correction().text
-                assert message_text is not None
-
             if message.moderation is not None:
                 message_text = get_moderation_text(
                     message.moderation.by, message.moderation.reason
                 )
 
+            elif message.retraction is not None:
+                message_text = get_retraction_text(message.retraction.timestamp)
+
+            elif corrected_message := message.get_last_correction():
+                message_text = corrected_message.text
+                assert message_text is not None
+
             # Nickname
             nickname = None
             if (
@@ -451,10 +453,12 @@ class ChatListRow(Gtk.ListBoxRow, SignalManager):
         if isinstance(self.contact, GroupchatParticipant):
             self.contact.room.disconnect_all_from_obj(self)
 
+        del self._menu_popover
         Gtk.ListBoxRow.do_unroot(self)
         app.check_finalize(self)
 
     def _on_close_button_clicked(self, _button: Gtk.Button) -> None:
+        self.reset_unread()
         app.window.activate_action(
             "win.remove-chat", GLib.Variant("as", [self.account, str(self.jid)])
         )
@@ -483,7 +487,6 @@ class ChatListRow(Gtk.ListBoxRow, SignalManager):
 
         self._menu_popover.set_menu_model(menu)
         self._menu_popover.set_pointing_to_coord(x=x, y=y)
-        self._connect(self._menu_popover, "closed", self._on_context_popover_closed)
         self.emit("context-menu-state-changed", True)
         self._menu_popover.popup()
 
@@ -491,11 +494,8 @@ class ChatListRow(Gtk.ListBoxRow, SignalManager):
         self.emit("context-menu-state-changed", False)
 
     def _on_prepare(
-        self, _drag_source: Gtk.DragSource, x: float, y: float
+        self, _drag_source: Gtk.DragSource, _x: float, _y: float
     ) -> Gdk.ContentProvider:
-        self._drag_hotspot_x = x
-        self._drag_hotspot_y = y
-
         value = GObject.Value()
         value.init(ChatListRow)
         value.set_object(self)
@@ -504,11 +504,14 @@ class ChatListRow(Gtk.ListBoxRow, SignalManager):
 
     def _on_drag_begin(self, _drag_source: Gtk.DragSource, drag: Gdk.Drag) -> None:
         assert not isinstance(self.contact, ResourceContact)
+
+        is_LTR = bool(self.get_direction() == Gtk.TextDirection.LTR)
+
         Gtk.DragIcon.set_from_paintable(
             drag,
             Gtk.WidgetPaintable().new(self),
-            int(self._drag_hotspot_x),
-            int(self._drag_hotspot_y),
+            hot_x=0 if is_LTR else self.get_width(),
+            hot_y=0,
         )
 
         app.window.highlight_dnd_targets(self, True)
@@ -571,12 +574,12 @@ class ChatListRow(Gtk.ListBoxRow, SignalManager):
         if self.contact.is_joining:
             self._ui.connection_icon.set_from_icon_name("feather-refresh-cw-symbolic")
             self._ui.connection_icon.add_css_class("spin")
-            self._ui.connection_icon.add_css_class("dim-label")
+            self._ui.connection_icon.add_css_class("dimmed")
             self._ui.connection_icon.set_tooltip_text(_("Joining Group Chat…"))
             self._ui.connection_icon.set_visible(True)
         elif self.contact.is_not_joined or not self._client.state.is_available:
             self._ui.connection_icon.set_from_icon_name("feather-zap-symbolic")
-            self._ui.connection_icon.add_css_class("warning-color")
+            self._ui.connection_icon.add_css_class("warning")
             self._ui.connection_icon.set_tooltip_text(_("Not connected"))
             self._ui.connection_icon.set_visible(True)
 
@@ -589,7 +592,7 @@ class ChatListRow(Gtk.ListBoxRow, SignalManager):
         if signal_name == "mam-sync-started":
             self._ui.connection_icon.set_from_icon_name("feather-refresh-cw-symbolic")
             self._ui.connection_icon.add_css_class("spin")
-            self._ui.connection_icon.add_css_class("info-color")
+            self._ui.connection_icon.add_css_class("accent")
             self._ui.connection_icon.set_tooltip_text(_("Fetching messages…"))
             self._ui.connection_icon.set_visible(True)
 
@@ -599,7 +602,7 @@ class ChatListRow(Gtk.ListBoxRow, SignalManager):
 
         self._reset_connection_icon()
         self._ui.connection_icon.set_from_icon_name("feather-zap-symbolic")
-        self._ui.connection_icon.add_css_class("error-color")
+        self._ui.connection_icon.add_css_class("error")
         self._ui.connection_icon.set_tooltip_text(
             _("There has been an error while trying to fetch messages: %s") % error_text
         )
@@ -608,10 +611,10 @@ class ChatListRow(Gtk.ListBoxRow, SignalManager):
     def _reset_connection_icon(self) -> None:
         self._ui.connection_icon.set_visible(False)
         self._ui.connection_icon.remove_css_class("spin")
-        self._ui.connection_icon.remove_css_class("dim-label")
-        self._ui.connection_icon.remove_css_class("info-color")
-        self._ui.connection_icon.remove_css_class("warning-color")
-        self._ui.connection_icon.remove_css_class("error-color")
+        self._ui.connection_icon.remove_css_class("dimmed")
+        self._ui.connection_icon.remove_css_class("accent")
+        self._ui.connection_icon.remove_css_class("warning")
+        self._ui.connection_icon.remove_css_class("error")
 
     def _on_muc_user_update(
         self, _contact: GroupchatParticipant, _signal_name: str, *args: Any
