@@ -21,6 +21,7 @@ from nbxmpp.modules.security_labels import SecurityLabel
 
 from gajim.common import app
 from gajim.common import configpaths
+from gajim.common import events
 from gajim.common import ged
 from gajim.common.client import Client
 from gajim.common.commands import CommandFailed
@@ -39,9 +40,9 @@ from gajim.common.storage.archive import models as mod
 from gajim.common.structs import ReplyData
 from gajim.common.types import ChatContactT
 
+from gajim.gtk.alert import InformationAlertDialog
 from gajim.gtk.builder import get_builder
 from gajim.gtk.chat_state_indicator import ChatStateIndicator
-from gajim.gtk.dialogs import SimpleDialog
 from gajim.gtk.emoji_chooser import EmojiChooser
 from gajim.gtk.menus import get_encryption_menu
 from gajim.gtk.menus import get_format_menu
@@ -140,13 +141,16 @@ class MessageActionsBox(Gtk.Grid, EventHelper, SignalManager):
             self._ui.request_voice_button, "clicked", self._on_request_voice_clicked
         )
 
-        self._connect_actions()
-
         app.plugin_manager.gui_extension_point(
             "message_actions_box", self, self._ui.action_box
         )
 
-        self.register_events([("message-sent", ged.GUI2, self._on_message_sent)])
+        self.register_events(
+            [
+                ("message-sent", ged.GUI2, self._on_message_sent),
+                ("register-actions", ged.GUI2, self._on_register_actions),
+            ]
+        )
 
     def do_unroot(self) -> None:
         self._disconnect_all()
@@ -160,6 +164,34 @@ class MessageActionsBox(Gtk.Grid, EventHelper, SignalManager):
         Gtk.Grid.do_unroot(self)
         app.check_finalize(self)
 
+    def _on_register_actions(self, _event: events.RegisterActions) -> None:
+        actions = [
+            "input-bold",
+            "input-italic",
+            "input-strike",
+            "input-clear",
+            "show-emoji-chooser",
+            "paste-as-quote",
+            "paste-as-code-block",
+            "quote",
+            "mention",
+            "reply",
+            "correct-message",
+        ]
+
+        for action in actions:
+            action = app.window.get_action(action)
+            self._connect(action, "activate", self._on_action)
+
+        action = app.window.get_action("set-encryption")
+        self._connect(action, "change-state", self._change_encryption)
+
+        # action = app.window.get_action('send-file-jingle')
+        # self._connect(action, 'notify::enabled', self._on_send_file_enabled_changed)
+
+        action = app.window.get_action("send-file-httpupload")
+        self._connect(action, "notify::enabled", self._on_send_file_enabled_changed)
+
     def get_current_contact(self) -> ChatContactT:
         assert self._contact is not None
         return self._contact
@@ -225,34 +257,7 @@ class MessageActionsBox(Gtk.Grid, EventHelper, SignalManager):
 
     def _on_emoji_picked(self, _emoji_chooser: EmojiChooser, text: str) -> None:
         self.msg_textview.insert_text(text)
-
-    def _connect_actions(self) -> None:
-        actions = [
-            "input-bold",
-            "input-italic",
-            "input-strike",
-            "input-clear",
-            "show-emoji-chooser",
-            "paste-as-quote",
-            "paste-as-code-block",
-            "quote",
-            "mention",
-            "reply",
-            "correct-message",
-        ]
-
-        for action in actions:
-            action = app.window.get_action(action)
-            self._connect(action, "activate", self._on_action)
-
-        action = app.window.get_action("set-encryption")
-        self._connect(action, "change-state", self._change_encryption)
-
-        # action = app.window.get_action('send-file-jingle')
-        # self._connect(action, 'notify::enabled', self._on_send_file_enabled_changed)
-
-        action = app.window.get_action("send-file-httpupload")
-        self._connect(action, "notify::enabled", self._on_send_file_enabled_changed)
+        GLib.idle_add(self.msg_textview.grab_focus_delayed)
 
     def _on_action(
         self, action: Gio.SimpleAction, param: GLib.Variant | None
@@ -474,10 +479,12 @@ class MessageActionsBox(Gtk.Grid, EventHelper, SignalManager):
         if message is None or message.text is None:
             return
 
+        if message.moderation or message.retraction:
+            return
+
         self._set_correcting(last_message_id)
 
-        if message.corrections:
-            message = message.get_last_correction()
+        message = message.get_last_correction() or message
 
         self.msg_textview.start_correction(message)
         self._ui.edit_box.set_visible(True)
@@ -738,14 +745,14 @@ class MessageActionsBox(Gtk.Grid, EventHelper, SignalManager):
         self._ui.sendfile_button.set_tooltip_text(tooltip_text)
 
     def _on_buffer_changed(self, _message_input: MessageInputTextView) -> None:
-        assert self._contact
+        if self._contact is None:
+            return
 
         has_text = self.msg_textview.has_text
         app.window.get_action("send-message").set_enabled(
             allow_send_message(has_text, self._contact)
         )
 
-        assert self._contact is not None
         encryption_name = self._contact.settings.get("encryption")
 
         if has_text and encryption_name:
@@ -824,7 +831,7 @@ class MessageActionsBox(Gtk.Grid, EventHelper, SignalManager):
 
             if not app.account_is_available(self._contact.account):
                 # we are not connected
-                SimpleDialog(
+                InformationAlertDialog(
                     _("Not Connected"),
                     _("Your message can not be sent until you are connected."),
                 )
@@ -863,7 +870,7 @@ class MessageActionsBox(Gtk.Grid, EventHelper, SignalManager):
             log.warning("Could not determine clipboard mime types")
             return
 
-        if "image/png" in mime_types:
+        if "image/png" in mime_types or "image/bmp" in mime_types:
             self._ui.input_overlay.set_visible(True)
             self._ui.input_overlay_label.set_text(_("Processing image…"))
             clipboard.read_texture_async(None, self._on_clipboard_read_texture_finished)
@@ -875,13 +882,26 @@ class MessageActionsBox(Gtk.Grid, EventHelper, SignalManager):
             clipboard.read_value_async(
                 Gdk.FileList, 0, None, self._on_clipboard_read_value_finished
             )
+            return
+
+        log.info("Could not process clipboard mime type %s", mime_types)
 
     def _on_clipboard_read_value_finished(
         self,
         clipboard: Gdk.Clipboard,
         result: Gio.AsyncResult,
     ) -> None:
-        file_list = clipboard.read_value_finish(result)
+        try:
+            file_list = clipboard.read_value_finish(result)
+        except Exception as e:
+            formats = clipboard.get_formats()
+            mime_types = formats.get_mime_types()
+            InformationAlertDialog(
+                _("Pasting Content Failed"),
+                _("Error: %s (mime types: %s)") % (e, mime_types),
+            )
+            return
+
         if file_list is None or not isinstance(file_list, Gdk.FileList):
             log.info("No URIs pasted")
             return
@@ -897,7 +917,8 @@ class MessageActionsBox(Gtk.Grid, EventHelper, SignalManager):
         try:
             texture = clipboard.read_texture_finish(result)
         except GLib.Error as e:
-            log.error("Could not read clipboard content: %s", e)
+            self._ui.input_overlay.set_visible(False)
+            InformationAlertDialog(_("Pasting Content Failed"), _("Error: %s") % e)
             return
 
         if texture is None:
