@@ -30,13 +30,14 @@ from gajim.common.styling import PlainBlock
 from gajim.common.styling import process
 from gajim.common.types import ChatContactT
 
+from gajim.gtk.alert import InformationAlertDialog
 from gajim.gtk.completion.commands import CommandsCompletionProvider
 from gajim.gtk.completion.emoji import EmojiCompletionProvider
 from gajim.gtk.completion.nickname import NicknameCompletionProvider
 from gajim.gtk.completion.popover import CompletionPopover
 from gajim.gtk.const import MAX_MESSAGE_LENGTH
 from gajim.gtk.menus import get_message_input_extra_context_menu
-from gajim.gtk.util.misc import scroll_to_end
+from gajim.gtk.util.misc import scroll_to
 from gajim.gtk.widgets import GdkRectangle
 
 if app.is_installed("SPELLING") or typing.TYPE_CHECKING:
@@ -63,11 +64,6 @@ class MessageInputTextView(GtkSource.View):
             None,
             (),
         ),
-        "line-count-changed": (
-            GObject.SignalFlags.RUN_LAST | GObject.SignalFlags.ACTION,
-            None,
-            (int,),
-        ),
     }
 
     def __init__(self, parent: Gtk.Widget) -> None:
@@ -77,6 +73,7 @@ class MessageInputTextView(GtkSource.View):
             wrap_mode=Gtk.WrapMode.WORD_CHAR,
             margin_top=3,
             margin_bottom=3,
+            valign=Gtk.Align.CENTER,
         )
 
         self._parent = parent
@@ -91,8 +88,6 @@ class MessageInputTextView(GtkSource.View):
 
         self._contact: ChatContactT | None = None
 
-        self._line_count = 1
-
         self._text_buffer_manager = TextBufferManager(self)
         self._text_buffer_manager.connect("buffer-changed", self._on_buffer_changed)
 
@@ -156,27 +151,6 @@ class MessageInputTextView(GtkSource.View):
             menu.append_section(_("Spell Checking"), self._speller_menu)
         self.set_extra_menu(menu)
 
-    def _count_lines(self) -> int:
-        """
-        Return the number of visible lines, which determines the
-        height of the textview.
-
-        The number of visible lines is the sum of
-        line breaks due to new line characters and word/char wraps
-        at the view's boundary.
-
-        We cannot use Gtk.TextBuffer.get_line_count() as it only
-        counts line breaks as defined above.
-        """
-        buf = self.get_buffer()
-        start_iter = buf.get_start_iter()
-        line_count = 1
-
-        while self.forward_display_line(start_iter):
-            line_count += 1
-
-        return line_count
-
     def _on_buffer_changed(self, _text_buffer_manager: TextBufferManager) -> None:
 
         buf = self.get_buffer()
@@ -191,11 +165,6 @@ class MessageInputTextView(GtkSource.View):
 
         self._on_text_changed()
 
-        line_count = self._count_lines()
-        if self._line_count != line_count:
-            self._line_count = line_count
-            self.emit("line-count-changed", self._line_count)
-
         self.emit("buffer-changed")
 
     @staticmethod
@@ -314,7 +283,7 @@ class MessageInputTextView(GtkSource.View):
         mark = buf.get_insert()
         iter_ = buf.get_iter_at_mark(mark)
         if buf.get_end_iter().equal(iter_):
-            GLib.idle_add(scroll_to_end, self.get_parent())
+            GLib.idle_add(scroll_to, self.get_parent(), "bottom")
 
     @property
     def has_text(self) -> bool:
@@ -334,7 +303,7 @@ class MessageInputTextView(GtkSource.View):
         mark = buf.get_insert()
         iter_ = buf.get_iter_at_mark(mark)
         if iter_.get_offset() == buf.get_end_iter().get_offset():
-            GLib.idle_add(scroll_to_end, textview.get_parent())
+            GLib.idle_add(scroll_to, textview.get_parent(), "bottom")
 
     def _get_active_iters(self) -> tuple[Gtk.TextIter, Gtk.TextIter]:
         buf = self.get_buffer()
@@ -428,7 +397,12 @@ class MessageInputTextView(GtkSource.View):
     def _on_clipboard_read_text_finished(
         self, clipboard: Gdk.Clipboard, result: Gio.AsyncResult, action_name: str
     ) -> None:
-        text = clipboard.read_text_finish(result)
+        try:
+            text = clipboard.read_text_finish(result)
+        except Exception as e:
+            InformationAlertDialog(_("Pasting Content Failed"), _("Error: %s") % e)
+            return
+
         if text is None:
             log.info("No text pasted")
             return
