@@ -75,9 +75,6 @@ class Plugins(Adw.PreferencesGroup, EventHelper, SignalManager):
         self.set_description(description_text)
         self.set_header_suffix(self._file_chooser_button)
 
-        self._load_installed_manifests()
-        self._load_repository_manifests()
-
         self.register_events(
             [
                 ("plugin-removed", ged.GUI1, self._on_plugin_removed),
@@ -92,6 +89,10 @@ class Plugins(Adw.PreferencesGroup, EventHelper, SignalManager):
             "plugin-updates-available", self._on_plugin_updates_available
         )
 
+        # Load manifests in idle fashion, since it blocks the UI
+        GLib.idle_add(self._load_installed_manifests)
+        GLib.idle_add(self._load_repository_manifests)
+
     def do_unroot(self) -> None:
         self._disconnect_all()
         app.plugin_repository.disconnect(self)
@@ -380,11 +381,14 @@ class PluginRow(Adw.ExpanderRow, SignalManager):
         return image
 
     def _get_error(self) -> tuple[bool, str]:
+        """Returns has_error, error_text"""
         if not self._installed:
             return False, ""
 
         plugin = app.plugin_manager.get_plugin(self._manifest.short_name)
-        assert plugin is not None
+        if plugin is None:
+            return True, _("Plugin could not be loaded")
+
         if not plugin.activatable:
             return True, plugin.available_text
         return False, ""
