@@ -31,11 +31,11 @@ from nbxmpp.structs import MuclumbusResult
 from nbxmpp.task import Task
 
 from gajim.common import app
-from gajim.common.configpaths import get_ui_path
 from gajim.common.const import AvatarSize
 from gajim.common.const import Direction
 from gajim.common.const import MUC_DISCO_ERRORS
-from gajim.common.const import URIType
+from gajim.common.const import PresenceShowExt
+from gajim.common.const import RFC5646_LANGUAGE_TAGS
 from gajim.common.helpers import to_user_string
 from gajim.common.i18n import _
 from gajim.common.i18n import get_rfc5646_lang
@@ -44,9 +44,10 @@ from gajim.common.modules.contacts import GroupchatContact
 from gajim.common.modules.util import as_task
 from gajim.common.util.jid import validate_jid
 from gajim.common.util.muc import get_group_chat_nick
-from gajim.common.util.text import get_country_flag_from_code
+from gajim.common.util.status import compare_show
 from gajim.common.util.text import to_one_line
 from gajim.common.util.uri import parse_uri
+from gajim.common.util.uri import XmppIri
 
 from gajim.gtk.builder import get_builder
 from gajim.gtk.chat_filter import ChatFilter
@@ -54,9 +55,11 @@ from gajim.gtk.chat_filter import ChatFilters
 from gajim.gtk.chat_filter import ChatTypeFilter
 from gajim.gtk.groupchat_info import GroupChatInfoScrolled
 from gajim.gtk.groupchat_nick import NickChooser
+from gajim.gtk.menus import get_start_chat_menu
 from gajim.gtk.menus import get_start_chat_row_menu
 from gajim.gtk.util.classes import SignalManager
 from gajim.gtk.util.icons import get_icon_theme
+from gajim.gtk.util.misc import get_ui_string
 
 # from gajim.gtk.tooltips import ContactTooltip
 from gajim.gtk.widgets import AccountBadge
@@ -111,6 +114,8 @@ class StartChatDialog(GajimAppWindow):
         self._connect(self._contact_view, "activate", self._on_contact_item_activated)
         self._ui.contact_scrolled.set_child(self._contact_view)
 
+        self._ui.settings_menu.set_menu_model(get_start_chat_menu())
+
         scale = self.get_scale_factor()
         self._add_accounts()
         self._add_contacts(scale)
@@ -190,6 +195,15 @@ class StartChatDialog(GajimAppWindow):
 
         self._contact_view.set_loading_finished()
 
+        action = Gio.SimpleAction.new_stateful(
+            "sort-by-show",
+            None,
+            GLib.Variant.new_boolean(app.settings.get("sort_by_show_in_start_chat")),
+        )
+        self._connect(action, "change-state", self._on_sort_by_show_changed)
+        self.window.add_action(action)
+
+        self._ui.search_entry.grab_focus()
         log.debug("Loading dialog finished")
         self.show()
 
@@ -293,6 +307,16 @@ class StartChatDialog(GajimAppWindow):
             self._new_contact_items[account] = item
             self._contact_view.add(item)
 
+    def _on_sort_by_show_changed(
+        self, action: Gio.SimpleAction, param: GLib.Variant
+    ) -> None:
+        action_state = action.get_state()
+        assert action_state is not None
+        new_state = not action_state.get_boolean()
+        app.settings.set("sort_by_show_in_start_chat", new_state)
+        action.set_state(GLib.Variant.new_boolean(new_state))
+        self._contact_view.invalidate_sort()
+
     def _on_page_changed(self, stack: Gtk.Stack, _param: Any) -> None:
         if stack.get_visible_child_name() == "account":
             self._ui.account_view.grab_focus()
@@ -380,8 +404,8 @@ class StartChatDialog(GajimAppWindow):
 
         search_text = search_entry.get_text()
         uri = parse_uri(search_text)
-        if uri.type == URIType.XMPP:
-            search_entry.set_text(uri.data["jid"])
+        if isinstance(uri, XmppIri):
+            search_entry.set_text(str(uri.jid))
             return
 
         if search_text:
@@ -499,7 +523,6 @@ class StartChatDialog(GajimAppWindow):
             return
 
         self._ui.stack.set_visible_child_name("progress")
-        self._ui.spinner.start()
         client = app.get_client(item.account)
         client.get_module("Discovery").disco_info(
             item.jid, callback=self._disco_info_received, user_data=item, timeout=10
@@ -522,7 +545,7 @@ class StartChatDialog(GajimAppWindow):
                 return
 
             # Handle other possible errors
-            self._show_error_page(error.get_text())
+            self._show_error_page(to_user_string(error))
             return
         except TimeoutStanzaError:
             # We reached the 10s timeout and we cannot
@@ -541,7 +564,6 @@ class StartChatDialog(GajimAppWindow):
 
     def _disco_muc(self, account: str, jid: JID, request_vcard: bool) -> None:
         self._ui.stack.set_visible_child_name("progress")
-        self._ui.spinner.start()
         client = app.get_client(account)
         client.get_module("Discovery").disco_muc(
             jid,
@@ -628,7 +650,7 @@ class StartChatDialog(GajimAppWindow):
         if button.get_active():
             self._chat_filter.reset()
             self._chat_filter.set_sensitive(False)
-            image.add_css_class("selected-color")
+            image.add_css_class("accent")
             self._ui.list_stack.set_visible_child_name("global")
 
             self._ui.global_search_placeholder_stack.set_visible_child_name(
@@ -642,7 +664,7 @@ class StartChatDialog(GajimAppWindow):
         else:
             self._chat_filter.set_sensitive(True)
             self._ui.search_entry.set_text("")
-            image.remove_css_class("selected-color")
+            image.remove_css_class("accent")
             self._ui.list_stack.set_visible_child_name("contacts")
             self._global_search_view.remove_all()
 
@@ -830,8 +852,8 @@ class ContactListView(BaseListView[type["ContactListItem"], type["ContactViewIte
         self._scroll_id = None
         self._search_string_list: list[str] = []
 
-        sorter = Gtk.CustomSorter.new(sort_func=self._sort_func)
-        self._sort_model = Gtk.SortListModel(sorter=sorter)
+        self._sorter = Gtk.CustomSorter.new(sort_func=self._sort_func)
+        self._sort_model = Gtk.SortListModel(sorter=self._sorter)
 
         self._custom_filter = Gtk.CustomFilter.new(self._filter_func)
 
@@ -847,6 +869,9 @@ class ContactListView(BaseListView[type["ContactListItem"], type["ContactViewIte
         self.set_model(self._selection_model)
 
     def do_unroot(self) -> None:
+        # The filter func needs to be unset before calling do_unroot (see #12213)
+        self._custom_filter.set_filter_func(None)
+        self._sorter.set_sort_func(None)
         Gtk.ListView.do_unroot(self)
         self._disconnect_all()
         app.check_finalize(self._model)
@@ -858,8 +883,8 @@ class ContactListView(BaseListView[type["ContactListItem"], type["ContactViewIte
         del self._filter_model
         del self._sort_model
         del self._selection_model
-        self._custom_filter.set_filter_func(None)
         del self._custom_filter
+        del self._sorter
         app.check_finalize(self)
 
     def set_loading_finished(self) -> None:
@@ -868,6 +893,9 @@ class ContactListView(BaseListView[type["ContactListItem"], type["ContactViewIte
     def get_count(self) -> int:
         return self._model.get_n_items()
 
+    def invalidate_sort(self) -> None:
+        self._sorter.changed(Gtk.SorterChange.DIFFERENT)
+
     def _on_filter_items_changed(
         self, filter_model: Gtk.FilterListModel, _pos: int, _removed: int, _added: int
     ) -> None:
@@ -892,6 +920,10 @@ class ContactListView(BaseListView[type["ContactListItem"], type["ContactViewIte
         self._scroll_id = GLib.timeout_add(50, _scroll_to)
 
     def _filter_func(self, item: ContactListItem) -> bool:
+        account = self._chat_filters.account
+        if account is not None and account != item.account:
+            return False
+
         if item.is_new:
             return True
 
@@ -899,10 +931,6 @@ class ContactListView(BaseListView[type["ContactListItem"], type["ContactViewIte
             if search_string not in item.search_string:
                 return False
 
-        account = self._chat_filters.account
-        if account is not None and account != item.account:
-            return False
-
         group = self._chat_filters.group
         if group is not None and group not in item.groups:
             return False
@@ -923,10 +951,22 @@ class ContactListView(BaseListView[type["ContactListItem"], type["ContactViewIte
         obj2: Any,
         _user_data: object | None,
     ) -> int:
-        if obj1.is_new:
-            return 1
 
-        return locale.strcoll(obj1.name, obj2.name)
+        if obj1.is_new != obj2.is_new:
+            return 1 if obj1.is_new else -1
+
+        if obj1.is_groupchat != obj2.is_groupchat:
+            return 1 if obj1.is_groupchat else -1
+
+        if obj1.is_self != obj2.is_self:
+            return 1 if obj1.is_self else -1
+
+        if app.settings.get("sort_by_show_in_start_chat"):
+            res = compare_show(obj1.show, obj2.show)
+            if res != 0:
+                return res
+
+        return locale.strcoll(obj1.name.lower(), obj2.name.lower())
 
     def add(self, item: ContactViewItem) -> None:
         self._model.append(item)
@@ -961,8 +1001,10 @@ class ContactListItem(GObject.Object):
     groups = GObject.Property(type=object)
     is_groupchat = GObject.Property(type=bool, default=False)
     is_new = GObject.Property(type=bool, default=False)
+    is_self = GObject.Property(type=bool, default=False)
     avatar_paintable = GObject.Property(type=Gdk.Paintable)
     idle = GObject.Property(type=object)
+    show = GObject.Property(type=object)
     status = GObject.Property(type=str)
     status_visible = GObject.Property(type=bool, default=False)
     search_string = GObject.Property(type=str)
@@ -984,14 +1026,16 @@ class ContactListItem(GObject.Object):
         idle = None
         status = ""
         groups = []
+        show = PresenceShowExt.OFFLINE
+        is_self = False
         if contact is not None and not groupchat:
             groups = sorted(contact.groups)
             status = to_one_line(contact.status)
             idle = contact.idle_datetime
+            show = contact.show
+            is_self = contact.is_self
 
-        menu = None
-        if groupchat:
-            menu = get_start_chat_row_menu(account, jid)
+        menu = get_start_chat_row_menu(account, jid)
 
         is_new = jid is None
 
@@ -1020,6 +1064,8 @@ class ContactListItem(GObject.Object):
             idle=idle,
             is_groupchat=groupchat,
             is_new=jid is None,
+            is_self=is_self,
+            show=show,
             status=status,
             status_visible=bool(status),
             groups=groups,
@@ -1032,12 +1078,13 @@ class ContactListItem(GObject.Object):
         return f"ContactListItem: {self.props.account} - {self.props.jid}"
 
 
-@Gtk.Template(filename=get_ui_path("contact_view_item.ui"))
+@Gtk.Template(string=get_ui_string("contact_view_item.ui"))
 class ContactViewItem(Gtk.Grid, SignalManager):
     __gtype_name__ = "ContactViewItem"
 
     _avatar: Gtk.Label = Gtk.Template.Child()
     _name_label: Gtk.Label = Gtk.Template.Child()
+    _address_label: Gtk.Label = Gtk.Template.Child()
     _status_label: Gtk.Label = Gtk.Template.Child()
     _idle_badge: IdleBadge = Gtk.Template.Child()
     _account_badge: AccountBadge = Gtk.Template.Child()
@@ -1064,6 +1111,7 @@ class ContactViewItem(Gtk.Grid, SignalManager):
     def bind(self, obj: GlobalListItem) -> None:
         bind_spec = [
             ("name", self._name_label, "label"),
+            ("jid", self._address_label, "label"),
             ("status", self._status_label, "label"),
             ("status_visible", self._status_label, "visible"),
             ("avatar_paintable", self._avatar, "paintable"),
@@ -1164,40 +1212,37 @@ class GlobalListItem(GObject.Object):
     name = GObject.Property(type=str)
     nusers = GObject.Property(type=str)
     description = GObject.Property(type=str)
+    language_visible = GObject.Property(type=bool, default=False)
     language = GObject.Property(type=str)
     language_code = GObject.Property(type=str)
 
     def __init__(self, item: MuclumbusItem) -> None:
         jid = JID.from_string(item.jid)
         name = item.name or jid.localpart or str(jid)
-
-        language_code = item.language.upper()[:2]
-        if language_code == "EN":
-            # Fix for most used languages/countries
-            language_code = "GB"
-
-        language_code = get_country_flag_from_code(language_code)
+        language = RFC5646_LANGUAGE_TAGS.get(item.language, item.language) or None
 
         super().__init__(
             jid=item.jid,
             name=name,
             nusers=item.nusers,
             description=item.description,
-            language=_("Language: %s") % item.language,
-            language_code=language_code,
+            language_visible=bool(language is not None),
+            language=_("Language: %s") % language,
+            language_code=item.language.upper()[:2],
         )
 
     def __repr__(self) -> str:
         return f"GlobalListItem: {self.props.jid} {self.props.name}"
 
 
-@Gtk.Template(filename=get_ui_path("global_view_item.ui"))
+@Gtk.Template(string=get_ui_string("global_view_item.ui"))
 class GlobalViewItem(Gtk.Box):
     __gtype_name__ = "GlobalViewItem"
 
     _name_box: Gtk.Box = Gtk.Template.Child()
     _name_label: Gtk.Label = Gtk.Template.Child()
     _description_label: Gtk.Label = Gtk.Template.Child()
+    _language_box: Gtk.Box = Gtk.Template.Child()
     _language_label: Gtk.Label = Gtk.Template.Child()
     _users_count_label: Gtk.Label = Gtk.Template.Child()
 
@@ -1210,8 +1255,9 @@ class GlobalViewItem(Gtk.Box):
             ("name", self._name_label, "label"),
             ("description", self._description_label, "label"),
             ("jid", self._name_box, "tooltip_text"),
+            ("language_visible", self._language_box, "visible"),
+            ("language", self._language_box, "tooltip_text"),
             ("language_code", self._language_label, "label"),
-            ("language", self._language_label, "tooltip_text"),
             ("nusers", self._users_count_label, "label"),
         ]
 
